@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2013,2022 Red Hat.
  * Copyright (c) 1997 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -10,14 +11,10 @@
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
- * You should have received a copy of the GNU General Public License along
- * with this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include "pmapi.h"
-#include "impl.h"
+#include "libpcp.h"
 #include "pmda.h"
 
 /* yacc/lex routines */
@@ -34,16 +31,18 @@ extern void setup_context(void);
 extern void reset_profile(void);
 extern char *strcons(char *, char *);
 extern char *strnum(int);
+extern char *strcluster(pmID);
 extern void initmetriclist(void);
 extern void addmetriclist(pmID);
 extern void initarglist(void);
 extern void addarglist(char *);
 extern void doargs(void);
-extern void printindom(FILE *, __pmInResult *);
+extern void printindom(FILE *, pmInResult *);
 extern void dohelp(int, int);
 extern void dostatus(void);
-extern int fillResult(pmResult *, int);
+extern int fillValues(pmValueSet *, int);
 extern void _dbDumpResult(FILE *, pmResult *, pmDesc *);
+extern void _dbPrintResult(FILE *, __pmResult *, pmDesc *);
 
 /* pmda exerciser routines */
 extern void opendso(char *, char *, int);
@@ -53,7 +52,9 @@ extern void openpmda(char *);
 extern void closepmda(void);
 extern void dopmda(int);
 extern void watch(char *);
-extern void opensocket(char *);
+extern void open_unix_socket(char *);
+extern void open_inet_socket(int);
+extern void open_ipv6_socket(int);
 
 /*
  * connection states
@@ -78,7 +79,7 @@ typedef struct {
 extern param_t	param;
 
 /* the single profile */
-extern __pmProfile	*profile;
+extern pmProfile	*profile;
 extern int		profile_changed;
 
 /* status info */
@@ -94,6 +95,20 @@ extern int timer;
 /* get descriptor for fetch or not */
 extern int get_desc;
 
+/* report instance names in pmResult, or not (use ???) */
+extern int get_iname;
+
 /* namespace pathnames */
 extern char *pmnsfile;
 extern char *cmd_namespace;
+
+/* garbage collection */
+extern void gc_add(void *);
+extern void gc_free(void);
+
+/* pid for daemon PMDA */
+extern pid_t pid;
+
+/* helper routines for instance id -> name lookup */
+extern char *dopmda_iname(pmInDom, int);
+extern char *dodso_iname(pmInDom, int);
