@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2021 Red Hat.
+ * Copyright (c) 2017-2021,2024 Red Hat.
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -23,49 +23,49 @@
 #if defined(HAVE_LIBUV)
 #include <hiredis-cluster/adapters/libuv.h>
 #else
-static int redisClusterLibuvAttach() { return REDIS_OK; }
+static int keyClusterLibuvAttach() { return RESP_OK; }
 #endif
 
 static char default_server[] = "localhost:6379";
 
 static void
-redis_connect_callback(const redisAsyncContext *redis, int status)
+key_server_connect_callback(const keysAsyncContext *keys, int status)
 {
-    if (status == REDIS_OK) {
+    if (status == RESP_OK) {
 	if (pmDebugOptions.series)
-	    fprintf(stderr, "Connected to Redis on %s:%d\n",
-			redis->c.tcp.host, redis->c.tcp.port);
-	redisAsyncEnableKeepAlive((redisAsyncContext *)redis);
-	/* TODO: if SSL inject redisSecureConnection() here */
+	    fprintf(stderr, "Connected to key server on %s:%d\n",
+			keys->c.tcp.host, keys->c.tcp.port);
+	keysAsyncEnableKeepAlive((keysAsyncContext *)keys);
+	/* TODO: client SSL? inject keysSecureConnection() here */
     } else if (pmDebugOptions.series) {
-	if (redis->c.connection_type == REDIS_CONN_UNIX)
+	if (keys->c.connection_type == RESP_CONN_UNIX)
 	    fprintf(stderr, "Connecting to %s failed - %s\n",
-			redis->c.unix_sock.path, redis->errstr);
+			keys->c.unix_sock.path, keys->errstr);
 	else
 	    fprintf(stderr, "Connecting to %s:%d failed - %s\n",
-			redis->c.tcp.host, redis->c.tcp.port, redis->errstr);
+			keys->c.tcp.host, keys->c.tcp.port, keys->errstr);
     }
 }
 
 static void
-redis_disconnect_callback(const redisAsyncContext *redis, int status)
+key_server_disconnect_callback(const keysAsyncContext *keys, int status)
 {
-    if (status == REDIS_OK) {
+    if (status == RESP_OK) {
 	if (pmDebugOptions.series)
-	    fprintf(stderr, "Disconnected from redis on %s:%d\n",
-			redis->c.tcp.host, redis->c.tcp.port);
+	    fprintf(stderr, "Disconnected from key server on %s:%d\n",
+			keys->c.tcp.host, keys->c.tcp.port);
     } else if (pmDebugOptions.series) {
-	if (redis->c.connection_type == REDIS_CONN_UNIX)
+	if (keys->c.connection_type == RESP_CONN_UNIX)
 	    fprintf(stderr, "Disconnecting from %s failed - %s\n",
-			redis->c.unix_sock.path, redis->errstr);
+			keys->c.unix_sock.path, keys->errstr);
 	else
 	    fprintf(stderr, "Disconnecting from %s:%d failed - %s\n",
-			redis->c.tcp.host, redis->c.tcp.port, redis->errstr);
+			keys->c.tcp.host, keys->c.tcp.port, keys->errstr);
     }
 }
 
 void
-redisSlotsSetupMetrics(redisSlots *slots)
+keySlotsSetupMetrics(keySlots *slots)
 {
     pmAtomValue	**table;
     pmUnits	units_count = MMV_UNITS(0, 0, 1, 0, 0, PM_COUNT_ONE);
@@ -79,47 +79,47 @@ redisSlotsSetupMetrics(redisSlots *slots)
     mmv_stats_add_metric(slots->registry, "requests.total", 1,
 	MMV_TYPE_U64, MMV_SEM_COUNTER, units_count, MMV_INDOM_NULL,
 	"number of requests",
-	"Total number of Redis requests sent");
+	"Total number of key server requests sent");
 
     mmv_stats_add_metric(slots->registry, "requests.error", 2,
 	MMV_TYPE_U64, MMV_SEM_COUNTER, units_count, MMV_INDOM_NULL,
 	"number of request errors",
-	"Total number of Redis request errors");
+	"Total number of key server request errors");
 
     mmv_stats_add_metric(slots->registry, "responses.total", 3,
 	MMV_TYPE_U64, MMV_SEM_COUNTER, units_count, MMV_INDOM_NULL,
 	"number of responses",
-	"Total number of Redis responses received");
+	"Total number of key server responses received");
 
     mmv_stats_add_metric(slots->registry, "responses.error", 4,
 	MMV_TYPE_U64, MMV_SEM_COUNTER, units_count, MMV_INDOM_NULL,
 	"number of error responses",
-	"Total number of Redis error responses received");
+	"Total number of key server error responses received");
 
     mmv_stats_add_metric(slots->registry, "responses.time", 5,
 	MMV_TYPE_U64, MMV_SEM_COUNTER, units_us, MMV_INDOM_NULL,
 	"total time for responses",
-	"Cumulative time taken to receive all Redis responses");
+	"Cumulative time taken to receive all key server responses");
 
     mmv_stats_add_metric(slots->registry, "requests.inflight.total", 6,
 	MMV_TYPE_U64, MMV_SEM_INSTANT, units_count, MMV_INDOM_NULL,
 	"inflight requests",
-	"Total number of inflight Redis requests");
+	"Total number of inflight key server requests");
 
     mmv_stats_add_metric(slots->registry, "requests.inflight.bytes", 7,
 	MMV_TYPE_U64, MMV_SEM_INSTANT, units_bytes, MMV_INDOM_NULL,
 	"bytes allocated for inflight requests",
-	"Memory currently allocated for inflight Redis requests");
+	"Memory currently allocated for inflight key server requests");
 
     mmv_stats_add_metric(slots->registry, "requests.total_bytes", 8,
 	MMV_TYPE_U64, MMV_SEM_COUNTER, units_bytes, MMV_INDOM_NULL,
 	"total bytes sent for requests",
-	"Cumulative count of bytes sent for all Redis requests");
+	"Cumulative count of bytes sent for all key server requests");
 
     mmv_stats_add_metric(slots->registry, "responses.total_bytes", 9,
 	MMV_TYPE_U64, MMV_SEM_COUNTER, units_bytes, MMV_INDOM_NULL,
 	"total bytes received in responses",
-	"Cumulative count of bytes received in Redis responses");
+	"Cumulative count of bytes received in key server responses");
 
     slots->map = map = mmv_stats_start(slots->registry);
 
@@ -145,7 +145,7 @@ redisSlotsSetupMetrics(redisSlots *slots)
 }
 
 int
-redisSlotsSetMetricRegistry(redisSlots *slots, mmv_registry_t *registry)
+keySlotsSetMetricRegistry(keySlots *slots, mmv_registry_t *registry)
 {
     if (slots) {
 	slots->registry = registry;
@@ -154,10 +154,10 @@ redisSlotsSetMetricRegistry(redisSlots *slots, mmv_registry_t *registry)
     return -ENOMEM;
 }
 
-redisSlots *
-redisSlotsInit(dict *config, void *events)
+keySlots *
+keySlotsInit(dict *config, void *events)
 {
-    redisSlots		*slots;
+    keySlots		*slots;
     sds			servers = NULL;
     sds			def_servers = NULL;
     sds			username = NULL;
@@ -166,9 +166,9 @@ redisSlotsInit(dict *config, void *events)
     struct timeval	connection_timeout = {5, 0}; // 5s
     struct timeval	command_timeout = {60, 0}; // 1m
 
-    if ((slots = (redisSlots *)calloc(1, sizeof(redisSlots))) == NULL) {
-	pmNotifyErr(LOG_ERR, "%s: failed to allocate redisSlots\n",
-			"redisSlotsInit");
+    if ((slots = (keySlots *)calloc(1, sizeof(keySlots))) == NULL) {
+	pmNotifyErr(LOG_ERR, "%s: failed to allocate keySlots\n",
+			"keySlotsInit");
 	return NULL;
     }
 
@@ -177,42 +177,48 @@ redisSlotsInit(dict *config, void *events)
     slots->keymap = dictCreate(&sdsKeyDictCallBacks, "keymap");
     if (slots->keymap == NULL) {
 	pmNotifyErr(LOG_ERR, "%s: failed to allocate keymap\n",
-			"redisSlotsInit");
+			"keySlotsInit");
 	free(slots);
 	return NULL;
     }
 
-    servers = pmIniFileLookup(config, "redis", "servers");
+    servers = pmIniFileLookup(config, "keys", "servers");
     if (servers == NULL)
-	servers = pmIniFileLookup(config, "pmseries", "servers");
+	servers = pmIniFileLookup(config, "redis", "servers"); // back-compat
+    if (servers == NULL)
+	servers = pmIniFileLookup(config, "pmseries", "servers"); // back-compat
     if (servers == NULL)
 	servers = def_servers = sdsnew(default_server);
 
-    username = pmIniFileLookup(config, "redis", "username");
+    username = pmIniFileLookup(config, "keys", "username");
+    if (username == NULL)
+	username = pmIniFileLookup(config, "redis", "username"); // back-compat
     if (username == NULL)
 	username = pmIniFileLookup(config, "pmseries", "auth.username");
-    password = pmIniFileLookup(config, "redis", "password");
+    password = pmIniFileLookup(config, "keys", "password");
+    if (password == NULL)
+	password = pmIniFileLookup(config, "redis", "password"); // back-compat
     if (password == NULL)
 	password = pmIniFileLookup(config, "pmseries", "auth.password");
 
-    if ((slots->acc = redisClusterAsyncContextInit()) == NULL) {
+    if ((slots->acc = keyClusterAsyncContextInit()) == NULL) {
 	/* Coverity CID370635 */
 	pmNotifyErr(LOG_ERR, "%s: %s failed\n",
-			"redisSlotsInit", "redisClusterAsyncContextInit");
+			"keySlotsInit", "keyClusterAsyncContextInit");
 	sdsfree(def_servers);
 	return slots;
     }
 
     if (slots->acc->err) {
-        pmNotifyErr(LOG_ERR, "%s: %s\n", "redisSlotsInit", slots->acc->errstr);
+        pmNotifyErr(LOG_ERR, "%s: %s\n", "keySlotsInit", slots->acc->errstr);
 	sdsfree(def_servers);
 	return slots;
     }
 
-    sts = redisClusterSetOptionAddNodes(slots->acc->cc, servers);
-    if (sts != REDIS_OK) {
-	pmNotifyErr(LOG_ERR, "%s: failed to add redis nodes: %s\n",
-			"redisSlotsInit", slots->acc->cc->errstr);
+    sts = keyClusterSetOptionAddNodes(slots->acc->cc, servers);
+    if (sts != RESP_OK) {
+	pmNotifyErr(LOG_ERR, "%s: failed to add key server nodes: %s\n",
+			"keySlotsInit", slots->acc->cc->errstr);
 	sdsfree(def_servers);
 	return slots;
     }
@@ -224,60 +230,60 @@ redisSlotsInit(dict *config, void *events)
      * fine
      */
     if (username != NULL && sdslen(username) > 0) {
-	sts = redisClusterSetOptionUsername(slots->acc->cc, username);
-	if (sts != REDIS_OK) {
-	    pmNotifyErr(LOG_ERR, "%s: failed to set Redis username: %s\n",
-		"redisClusterSetOptionUsername", slots->acc->cc->errstr);
+	sts = keyClusterSetOptionUsername(slots->acc->cc, username);
+	if (sts != RESP_OK) {
+	    pmNotifyErr(LOG_ERR, "%s: failed to set key server username: %s\n",
+		"keyClusterSetOptionUsername", slots->acc->cc->errstr);
 	    return slots;
 	}
     }
 
     /*
-     * see note above re empty configuration values
-     * having only a password set and no username is a valid Redis
-     * configuration, see https://redis.io/commands/auth
+     * see note above re empty configuration values having only a password
+     * set and no username is a valid key server configuration, details:
+     * https://valkey.io/commands/auth
      */
     if (password != NULL && sdslen(password) > 0) {
-	sts = redisClusterSetOptionPassword(slots->acc->cc, password);
-	if (sts != REDIS_OK) {
-	    pmNotifyErr(LOG_ERR, "%s: failed to set Redis password: %s\n",
-		"redisClusterSetOptionPassword", slots->acc->cc->errstr);
+	sts = keyClusterSetOptionPassword(slots->acc->cc, password);
+	if (sts != RESP_OK) {
+	    pmNotifyErr(LOG_ERR, "%s: failed to set key server password: %s\n",
+		"keyClusterSetOptionPassword", slots->acc->cc->errstr);
 	    return slots;
 	}
     }
 
-    sts = redisClusterSetOptionConnectTimeout(slots->acc->cc, connection_timeout);
-    if (sts != REDIS_OK) {
+    sts = keyClusterSetOptionConnectTimeout(slots->acc->cc, connection_timeout);
+    if (sts != RESP_OK) {
 	pmNotifyErr(LOG_ERR, "%s: failed to set connect timeout: %s\n",
-			"redisSlotsInit", slots->acc->errstr);
+			"keySlotsInit", slots->acc->errstr);
 	return slots;
     }
 
-    sts = redisClusterSetOptionTimeout(slots->acc->cc, command_timeout);
-    if (sts != REDIS_OK) {
+    sts = keyClusterSetOptionTimeout(slots->acc->cc, command_timeout);
+    if (sts != RESP_OK) {
 	pmNotifyErr(LOG_ERR, "%s: failed to set command timeout: %s\n",
-			"redisSlotsInit", slots->acc->cc->errstr);
+			"keySlotsInit", slots->acc->cc->errstr);
 	return slots;
     }
 
-    sts = redisClusterLibuvAttach(slots->acc, slots->events);
-    if (sts != REDIS_OK) {
+    sts = keyClusterLibuvAttach(slots->acc, slots->events);
+    if (sts != RESP_OK) {
 	pmNotifyErr(LOG_ERR, "%s: failed to attach to event loop: %s\n",
-			"redisSlotsInit", slots->acc->errstr);
+			"keySlotsInit", slots->acc->errstr);
 	return slots;
     }
 
-    sts = redisClusterAsyncSetConnectCallback(slots->acc, redis_connect_callback);
-    if (sts != REDIS_OK) {
+    sts = keyClusterAsyncSetConnectCallback(slots->acc, key_server_connect_callback);
+    if (sts != RESP_OK) {
 	pmNotifyErr(LOG_ERR, "%s: failed to set connect callback: %s\n",
-			"redisSlotsInit", slots->acc->errstr);
+			"keySlotsInit", slots->acc->errstr);
 	return slots;
     }
 
-    sts = redisClusterAsyncSetDisconnectCallback(slots->acc, redis_disconnect_callback);
-    if (sts != REDIS_OK) {
+    sts = keyClusterAsyncSetDisconnectCallback(slots->acc, key_server_disconnect_callback);
+    if (sts != RESP_OK) {
 	pmNotifyErr(LOG_ERR, "%s: failed to set disconnect callback: %s\n",
-			"redisSlotsInit", slots->acc->errstr);
+			"keySlotsInit", slots->acc->errstr);
 	return slots;
     }
 
@@ -286,11 +292,11 @@ redisSlotsInit(dict *config, void *events)
 
 /**
  * despite the name, this function also handles the initial
- * connection to Redis
+ * connection to the key server
  */
 void
-redisSlotsReconnect(redisSlots *slots, redisSlotsFlags flags,
-		redisInfoCallBack info, redisDoneCallBack done,
+keySlotsReconnect(keySlots *slots, keySlotsFlags flags,
+		keysInfoCallBack info, keysDoneCallBack done,
 		void *userdata, void *events, void *arg)
 {
     dictIterator	*iterator;
@@ -304,26 +310,26 @@ redisSlotsReconnect(redisSlots *slots, redisSlotsFlags flags,
     slots->state = SLOTS_CONNECTING;
     slots->conn_seq++;
 
-    /* reset Redis context in case of reconnect */
+    /* reset key server context in case of reconnect */
     if (slots->acc->err) {
 	/* reset possible 'Connection refused' error before reconnecting */
 	slots->acc->err = 0;
 	memset(slots->acc->errstr, '\0', strlen(slots->acc->errstr));
     }
-    redisClusterAsyncDisconnect(slots->acc);
+    keyClusterAsyncDisconnect(slots->acc);
 
-    /* reset redisSlots in case of reconnect */
+    /* reset keySlots in case of reconnect */
     slots->cluster = 0;
     slots->search = 0;
     dictEmpty(slots->keymap, NULL);
 
-    sts = redisClusterConnect2(slots->acc->cc);
-    if (sts == REDIS_OK) {
+    sts = keyClusterConnect2(slots->acc->cc);
+    if (sts == RESP_OK) {
 	slots->cluster = 1;
     }
     else if (slots->acc->cc->err &&
-		strcmp(slots->acc->cc->errstr, REDIS_ENOCLUSTER) == 0) {
-	/* Redis instance has cluster support disabled */
+		strcmp(slots->acc->cc->errstr, RESP_ENOCLUSTER) == 0) {
+	/* key server instance has cluster support disabled */
 	slots->acc->cc->err = 0;
 	memset(slots->acc->cc->errstr, '\0', strlen(slots->acc->cc->errstr));
 	slots->cluster = 0;
@@ -338,7 +344,7 @@ redisSlotsReconnect(redisSlots *slots, redisSlotsFlags flags,
 	if (entry && dictNext(iterator)) {
 	    dictReleaseIterator(iterator);
 	    pmNotifyErr(LOG_ERR, "%s: more than one node is configured, "
-			"but cluster mode is disabled", "redisSlotsReconnect");
+			"but cluster mode is disabled", "keySlotsReconnect");
 	    slots->state = SLOTS_ERR_FATAL;
 	    return;
 	}
@@ -346,7 +352,7 @@ redisSlotsReconnect(redisSlots *slots, redisSlotsFlags flags,
     }
     else {
 	if (log_connection_errors || pmDebugOptions.desperate) {
-	    pmNotifyErr(LOG_INFO, "Cannot connect to Redis: %s\n",
+	    pmNotifyErr(LOG_INFO, "Cannot connect to key server: %s\n",
 			slots->acc->cc->errstr);
 	    log_connection_errors = 0;
 	}
@@ -356,43 +362,44 @@ redisSlotsReconnect(redisSlots *slots, redisSlotsFlags flags,
 
     slots->state = SLOTS_CONNECTED;
     log_connection_errors = 1;
-    redisSchemaLoad(slots, flags, info, done, userdata, events, arg);
+    keysSchemaLoad(slots, flags, info, done, userdata, events, arg);
 }
 
 /**
- * this method allocates the redisSlots struct and exists for backwards
- * compatibility, the actual connection to Redis happens in
- * redisSlotsReconnect()
+ * this method allocates the keySlots struct and exists for backwards
+ * compatibility, the actual connection to the key server happens in
+ * keySlotsReconnect()
  */
-redisSlots *
-redisSlotsConnect(dict *config, redisSlotsFlags flags,
-		redisInfoCallBack info, redisDoneCallBack done,
+keySlots *
+keySlotsConnect(dict *config, keySlotsFlags flags,
+		keysInfoCallBack info, keysDoneCallBack done,
 		void *userdata, void *events, void *arg)
 {
-    redisSlots			*slots;
+    keySlots			*slots;
     sds				enabled, msg;
 
-    enabled = pmIniFileLookup(config, "redis", "enabled");
+    if (!(enabled = pmIniFileLookup(config, "resp", "enabled")))
+	enabled = pmIniFileLookup(config, "redis", "enabled"); // back-compat
     if (enabled && strcmp(enabled, "false") == 0)
 	return NULL;
 
-    slots = redisSlotsInit(config, events);
+    slots = keySlotsInit(config, events);
     if (slots == NULL) {
-	infofmt(msg, "Failed to allocate memory for Redis slots");
+	infofmt(msg, "Failed to allocate memory for key server slots");
 	info(PMLOG_ERROR, msg, arg);
 	sdsfree(msg);
 	return NULL;
     }
 
-    redisSlotsReconnect(slots, flags, info, done, userdata, events, arg);
+    keySlotsReconnect(slots, flags, info, done, userdata, events, arg);
     return slots;
 }
 
 void
-redisSlotsFree(redisSlots *slots)
+keySlotsFree(keySlots *slots)
 {
-    redisClusterAsyncDisconnect(slots->acc);
-    redisClusterAsyncFree(slots->acc);
+    keyClusterAsyncDisconnect(slots->acc);
+    keyClusterAsyncFree(slots->acc);
     dictRelease(slots->keymap);
     memset(slots, 0, sizeof(*slots));
     free(slots);
@@ -407,13 +414,13 @@ gettimeusec(void)
     return (uint64_t)now.tv_sec * 1000000 + (uint64_t)now.tv_usec;
 }
 
-static redisSlotsReplyData *
-redisSlotsReplyDataAlloc(redisSlots *slots, size_t req_size,
-			redisClusterCallbackFn *callback, void *arg)
+static keySlotsReplyData *
+keySlotsReplyDataAlloc(keySlots *slots, size_t req_size,
+			keyClusterCallbackFn *callback, void *arg)
 {
-    redisSlotsReplyData *srd;
+    keySlotsReplyData *srd;
 
-    srd = calloc(1, sizeof(redisSlotsReplyData));
+    srd = calloc(1, sizeof(keySlotsReplyData));
     if (srd == NULL) {
         return NULL;
     }
@@ -428,13 +435,13 @@ redisSlotsReplyDataAlloc(redisSlots *slots, size_t req_size,
 }
 
 static inline void
-redisSlotsReplyDataFree(redisSlotsReplyData *srd)
+keySlotsReplyDataFree(keySlotsReplyData *srd)
 {
     free(srd);
 }
 
 uint64_t
-redisSlotsInflightRequests(redisSlots *slots)
+keySlotsInflightRequests(keySlots *slots)
 {
     pmAtomValue		*atom;
 
@@ -443,10 +450,10 @@ redisSlotsInflightRequests(redisSlots *slots)
 }
 
 static void
-redisSlotsReplyCallback(redisClusterAsyncContext *c, void *r, void *arg)
+keySlotsReplyCallback(keyClusterAsyncContext *c, void *r, void *arg)
 {
-    redisSlotsReplyData *srd = arg;
-    redisReply 		*reply = r;
+    keySlotsReplyData *srd = arg;
+    respReply 		*reply = r;
     void		*map = srd->slots->map;
 
     if (map) {
@@ -470,16 +477,16 @@ redisSlotsReplyCallback(redisClusterAsyncContext *c, void *r, void *arg)
 	mmv_add(map, metrics[SLOT_RESPONSES_TOTAL_BYTES], &delta);
 	mmv_inc(map, metrics[SLOT_RESPONSES_TOTAL]);
 
-	if (reply == NULL || reply->type == REDIS_REPLY_ERROR)
+	if (reply == NULL || reply->type == RESP_REPLY_ERROR)
 	    mmv_inc(map, metrics[SLOT_RESPONSES_ERROR]);
     }
 
     /**
      * handle connection resets
      *
-     * Why here and not in redis_disconnect_callback?
-     * Access to redisSlots->state is required. We cannot save a pointer to
-     * redisSlots in redisAsyncContext->data, because this member is already
+     * Why here and not in key_server_disconnect_callback?
+     * Access to keySlots->state is required. We cannot save a pointer to
+     * keySlots in keysAsyncContext->data, because this member is already
      * used by hiredis-cluster
      *
      * fwiw, in case one node of a cluster is down, slots->state should not be
@@ -491,72 +498,74 @@ redisSlotsReplyCallback(redisClusterAsyncContext *c, void *r, void *arg)
      * hiredis, update the disconnect callback to return the cluster context and
      * use this new disconnect callback instead of the conditional below.
      *
-     * Register a Redis disconnect if:
-     * * Redis returns an I/O error. In this case errno is also set, but
+     * Register a server disconnect if:
+     * * The server returns an I/O error.  In this case errno is also set, but
      *   there are lots of different error codes for connection failures (for example
      *   ECONNRESET, ENETUNREACH, ENETDOWN, ...) - defensively assume all require a
      *   reconnect
-     * * Redis returns the "LOADING Redis is loading the dataset in memory" error
-     * * ignore any errors for Redis requests pre-dating the latest (current) Redis
-     *   connection (to handle the case where a Redis callback returns after a new
+     * * Server returns the "LOADING ... is loading the dataset in memory" error
+     * * Ignore any errors for server requests pre-dating the latest (current)
+     *   connection (to handle the case where a callback returns after a new
      *   connection was already established)
-     * * ignore any errors if the state is already set to SLOTS_DISCONNECTED
-     * * ignore errors if cluster mode is enabled (for now, will change in a later release)
+     * * Ignore any errors if the state is already set to SLOTS_DISCONNECTED
+     * * Ignore errors if cluster mode is enabled.
      */
-    if (((reply == NULL && c->err == REDIS_ERR_IO) ||
-         (reply != NULL && reply->type == REDIS_REPLY_ERROR && strcmp(reply->str, REDIS_ELOADING) == 0)) &&
+    if (((reply == NULL && c->err == RESP_ERR_IO) ||
+         (reply != NULL && reply->type == RESP_REPLY_ERROR &&
+	  (strncmp(reply->str, RESP_ELOADING, strlen(RESP_ELOADING)) == 0 &&
+	   strstr(reply->str, RESP_ELOADDATA) != NULL))) &&
 	srd->conn_seq == srd->slots->conn_seq &&
 	srd->slots->state != SLOTS_DISCONNECTED &&
 	srd->slots->cluster == 0) {
-	pmNotifyErr(LOG_ERR, "Lost connection to Redis.\n");
+	pmNotifyErr(LOG_ERR, "Lost connection to key server.\n");
 	srd->slots->state = SLOTS_DISCONNECTED;
     }
 
     srd->callback(c, r, srd->arg);
-    redisSlotsReplyDataFree(arg);
+    keySlotsReplyDataFree(arg);
 }
 
 /*
- * Submit an arbitrary request to a (set of) Redis instance(s).
+ * Submit an arbitrary request to a (set of) key server instance(s).
  * The given key is used to determine the slot used, as per the
- * cluster specification - https://redis.io/topics/cluster-spec
+ * cluster specification - https://valkey.io/topics/cluster-spec
  * 
- * Serves mainly as a wrapper to redisClusterAsyncFormattedCommand
+ * Serves mainly as a wrapper to keyClusterAsyncFormattedCommand
  * including debug output and error handling
  */
 int
-redisSlotsRequest(redisSlots *slots, const sds cmd,
-		redisClusterCallbackFn *callback, void *arg)
+keySlotsRequest(keySlots *slots, const sds cmd,
+		keyClusterCallbackFn *callback, void *arg)
 {
     int			sts;
     uint64_t		size;
-    redisSlotsReplyData	*srd;
+    keySlotsReplyData	*srd;
 
     /*
-     * redisSlotsSetupStart() also sends Redis requests,
+     * keySlotsSetupStart() also sends key server requests,
      * therefore both SLOTS_CONNECTED and SLOTS_READY states are valid
      */
     if (UNLIKELY(slots->state != SLOTS_CONNECTED && slots->state != SLOTS_READY))
 	return -ENOTCONN;
 
     if (!slots->cluster)
-	return redisSlotsRequestFirstNode(slots, cmd, callback, arg);
+	return keySlotsRequestFirstNode(slots, cmd, callback, arg);
 
     if (UNLIKELY(pmDebugOptions.desperate))
-	fprintf(stderr, "%s: sending raw redis command:\n%s",
-			"redisSlotsRequest", cmd);
+	fprintf(stderr, "%s: sending raw key server command:\n%s",
+			"keySlotsRequest", cmd);
 
     size = sdslen(cmd);
-    if ((srd = redisSlotsReplyDataAlloc(slots, size, callback, arg)) == NULL) {
+    if ((srd = keySlotsReplyDataAlloc(slots, size, callback, arg)) == NULL) {
 	mmv_inc(slots->map, slots->metrics[SLOT_REQUESTS_ERROR]);
 	pmNotifyErr(LOG_ERR, "%s: failed to allocate reply data (%llu bytes)\n",
-			"redisSlotsRequest", (unsigned long long)size);
+			"keySlotsRequest", (unsigned long long)size);
 	return -ENOMEM;
     }
-    if ((sts = redisClusterAsyncFormattedCommand(slots->acc,
-		    redisSlotsReplyCallback, srd, cmd, size)) != REDIS_OK) {
+    if ((sts = keyClusterAsyncFormattedCommand(slots->acc,
+		    keySlotsReplyCallback, srd, cmd, size)) != RESP_OK) {
 	mmv_inc(slots->map, slots->metrics[SLOT_REQUESTS_ERROR]);
-	pmNotifyErr(LOG_ERR, "%s: %s (%s)\n", "redisSlotsRequest",
+	pmNotifyErr(LOG_ERR, "%s: %s (%s)\n", "keySlotsRequest",
 			slots->acc->errstr, cmd);
 	return -ENOMEM;
     }
@@ -566,22 +575,22 @@ redisSlotsRequest(redisSlots *slots, const sds cmd,
     mmv_inc(slots->map, slots->metrics[SLOT_REQUESTS_INFLIGHT_TOTAL]);
     mmv_inc(slots->map, slots->metrics[SLOT_REQUESTS_TOTAL]);
 
-    return REDIS_OK;
+    return RESP_OK;
 }
 
 int
-redisSlotsRequestFirstNode(redisSlots *slots, const sds cmd,
-		redisClusterCallbackFn *callback, void *arg)
+keySlotsRequestFirstNode(keySlots *slots, const sds cmd,
+		keyClusterCallbackFn *callback, void *arg)
 {
     dictIterator	*iterator;
     dictEntry		*entry;
     cluster_node	*node;
-    redisSlotsReplyData	*srd;
+    keySlotsReplyData	*srd;
     uint64_t		size;
     int			sts;
 
     /*
-     * redisSlotsSetupStart() also sends Redis requests,
+     * keySlotsSetupStart() also sends key server requests,
      * therefore both SLOTS_CONNECTED and SLOTS_READY states are valid
      */
     if (UNLIKELY(slots->state != SLOTS_CONNECTED && slots->state != SLOTS_READY))
@@ -591,29 +600,29 @@ redisSlotsRequestFirstNode(redisSlots *slots, const sds cmd,
     entry = dictNext(iterator);
     dictReleaseIterator(iterator);
     if (!entry) {
-	pmNotifyErr(LOG_ERR, "%s: No Redis node configured.",
-			"redisSlotsRequestFirstNode");
-	return REDIS_ERR;
+	pmNotifyErr(LOG_ERR, "%s: No key server node configured.",
+			"keySlotsRequestFirstNode");
+	return RESP_ERR;
     }
 
     node = dictGetVal(entry);
     if (UNLIKELY(pmDebugOptions.desperate))
-	fprintf(stderr, "%s: sending raw redis command to node %s\n%s",
-			"redisSlotsRequestFirstNode", node->addr, cmd);
+	fprintf(stderr, "%s: sending raw key server command to node %s\n%s",
+			"keySlotsRequestFirstNode", node->addr, cmd);
 
     size = sdslen(cmd);
-    if ((srd = redisSlotsReplyDataAlloc(slots, size, callback, arg)) == NULL) {
+    if ((srd = keySlotsReplyDataAlloc(slots, size, callback, arg)) == NULL) {
 	mmv_inc(slots->map, slots->metrics[SLOT_REQUESTS_ERROR]);
 	pmNotifyErr(LOG_ERR, "%s: failed to allocate reply data (%llu bytes)",
-			"redisSlotsRequestFirstNode", (unsigned long long)size);
+			"keySlotsRequestFirstNode", (unsigned long long)size);
 	return -ENOMEM;
     }
-    sts = redisClusterAsyncFormattedCommandToNode(slots->acc, node,
-			redisSlotsReplyCallback, srd, cmd, size);
-    if (sts != REDIS_OK) {
+    sts = keyClusterAsyncFormattedCommandToNode(slots->acc, node,
+			keySlotsReplyCallback, srd, cmd, size);
+    if (sts != RESP_OK) {
 	mmv_inc(slots->map, slots->metrics[SLOT_REQUESTS_ERROR]);
 	pmNotifyErr(LOG_ERR, "%s: %s (%s)\n",
-			"redisSlotsRequestFirstNode", slots->acc->errstr, cmd);
+			"keySlotsRequestFirstNode", slots->acc->errstr, cmd);
 	return -ENOMEM;
     }
 
@@ -622,16 +631,16 @@ redisSlotsRequestFirstNode(redisSlots *slots, const sds cmd,
     mmv_inc(slots->map, slots->metrics[SLOT_REQUESTS_INFLIGHT_TOTAL]);
     mmv_inc(slots->map, slots->metrics[SLOT_REQUESTS_TOTAL]);
 
-    return REDIS_OK;
+    return RESP_OK;
 }
 
 int
-redisSlotsProxyConnect(redisSlots *slots, redisInfoCallBack info,
-	redisReader **readerp, const char *buffer, ssize_t nread,
-	redisClusterCallbackFn *callback, void *arg)
+keySlotsProxyConnect(keySlots *slots, keysInfoCallBack info,
+	respReader **readerp, const char *buffer, ssize_t nread,
+	keyClusterCallbackFn *callback, void *arg)
 {
-    redisReader		*reader = *readerp;
-    redisReply		*reply = NULL;
+    respReader		*reader = *readerp;
+    respReply		*reply = NULL;
     dictEntry		*entry;
     size_t		replyStartPosition;
     long long		position;
@@ -640,24 +649,24 @@ redisSlotsProxyConnect(redisSlots *slots, redisInfoCallBack info,
     int			sts;
 
     if (!reader &&
-	(reader = *readerp = redisReaderCreate()) == NULL) {
-	infofmt(msg, "out-of-memory for Redis client reader");
+	(reader = *readerp = respReaderCreate()) == NULL) {
+	infofmt(msg, "out-of-memory for key client reader");
 	info(PMLOG_REQUEST, msg, arg), sdsfree(msg);
 	return -ENOMEM;
     }
 
-    if (redisReaderFeed(reader, buffer, nread) != REDIS_OK) {
-	infofmt(msg, "failed to parse Redis protocol request");
+    if (respReaderFeed(reader, buffer, nread) != RESP_OK) {
+	infofmt(msg, "failed to parse RESP protocol request");
 	info(PMLOG_REQUEST, msg, arg), sdsfree(msg);
 	return -EPROTO;
     }
 
-    /* parse all Redis requests contained in buffer (Redis pipelining) */
+    /* parse all key server requests contained in buffer (pipelining) */
     while (1) {
 	replyStartPosition = reader->pos;
-	sts = redisReaderGetReply(reader, (void **)&reply);
-	if (sts != REDIS_OK) {
-	    infofmt(msg, "failed to parse Redis protocol request");
+	sts = respReaderGetReply(reader, (void **)&reply);
+	if (sts != RESP_OK) {
+	    infofmt(msg, "failed to parse RESP protocol request");
 	    info(PMLOG_REQUEST, msg, arg), sdsfree(msg);
 	    return -EPROTO;
 	}
@@ -667,9 +676,9 @@ redisSlotsProxyConnect(redisSlots *slots, redisInfoCallBack info,
 
 	cmd = NULL;
 	hasKey = 0;
-	if (reply->type == REDIS_REPLY_ARRAY ||
-	    reply->type == REDIS_REPLY_MAP ||
-	    reply->type == REDIS_REPLY_SET)
+	if (reply->type == RESP_REPLY_ARRAY ||
+	    reply->type == RESP_REPLY_MAP ||
+	    reply->type == RESP_REPLY_SET)
 	    cmd = sdsnew(reply->element[0]->str);
 	if (cmd && (entry = dictFind(slots->keymap, cmd)) != NULL) {
 	    position = dictGetSignedIntegerVal(entry);
@@ -680,14 +689,14 @@ redisSlotsProxyConnect(redisSlots *slots, redisInfoCallBack info,
 
 	cmd = sdsnewlen(reader->buf + replyStartPosition, reader->pos - replyStartPosition);
 	if (hasKey)
-	    sts = redisSlotsRequest(slots, cmd, callback, arg);
+	    sts = keySlotsRequest(slots, cmd, callback, arg);
 	else
-	    sts = redisSlotsRequestFirstNode(slots, cmd, callback, arg);
+	    sts = keySlotsRequestFirstNode(slots, cmd, callback, arg);
 	sdsfree(cmd);
 
-	if (sts != REDIS_OK) {
-	    redisReply *errorReply = calloc(1, sizeof(redisReply));
-	    errorReply->type = REDIS_REPLY_ERROR;
+	if (sts != RESP_OK) {
+	    respReply *errorReply = calloc(1, sizeof(respReply));
+	    errorReply->type = RESP_REPLY_ERROR;
 	    errorReply->str = slots->acc->errstr;
 	    errorReply->len = strlen(slots->acc->errstr);
 	    callback(slots->acc, errorReply, arg);
@@ -697,47 +706,47 @@ redisSlotsProxyConnect(redisSlots *slots, redisInfoCallBack info,
 }
 
 void
-redisSlotsProxyFree(redisReader *reader)
+keySlotsProxyFree(respReader *reader)
 {
     if (reader)
-	redisReaderFree(reader);
+	respReaderFree(reader);
 }
 
 /*
- * Helper routines for handling various expected Redis reply types.
+ * Helper routines for handling various expected RESP reply types.
  */
 
 int
-testReplyError(redisReply *reply, const char *server_message)
+testReplyError(respReply *reply, const char *server_message)
 {
-    return (reply && reply->type == REDIS_REPLY_ERROR &&
+    return (reply && reply->type == RESP_REPLY_ERROR &&
 	    strcmp(reply->str, server_message) == 0);
 }
 
 void
-reportReplyError(redisInfoCallBack info, void *userdata,
-	redisClusterAsyncContext *acc, redisReply *reply, const char *format, va_list argp)
+reportReplyError(keysInfoCallBack info, void *userdata,
+	keyClusterAsyncContext *acc, respReply *reply, const char *format, va_list argp)
 {
     sds			msg;
 
     msg = sdscatvprintf(sdsempty(), format, argp);
-    if (reply && reply->type == REDIS_REPLY_ERROR)
-	msg = sdscatfmt(msg, "\nRedis reply error: %s", reply->str);
+    if (reply && reply->type == RESP_REPLY_ERROR)
+	msg = sdscatfmt(msg, "\nRESP reply error: %s", reply->str);
     else if (acc->err)
-	msg = sdscatfmt(msg, "\nRedis acc error: %s", acc->errstr);
+	msg = sdscatfmt(msg, "\nRESP acc error: %s", acc->errstr);
     else if (acc->cc->err)
-	msg = sdscatfmt(msg, "\nRedis cc error: %s", acc->cc->errstr);
+	msg = sdscatfmt(msg, "\nRESP cc error: %s", acc->cc->errstr);
     info(PMLOG_RESPONSE, msg, userdata);
     sdsfree(msg);
 }
 
 int
-checkStatusReplyOK(redisInfoCallBack info, void *userdata,
-		redisClusterAsyncContext *acc, redisReply *reply, const char *format, ...)
+checkStatusReplyOK(keysInfoCallBack info, void *userdata,
+		keyClusterAsyncContext *acc, respReply *reply, const char *format, ...)
 {
     va_list		argp;
 
-    if (reply && reply->type == REDIS_REPLY_STATUS &&
+    if (reply && reply->type == RESP_REPLY_STATUS &&
 	(strcmp("OK", reply->str) == 0 || strcmp("QUEUED", reply->str) == 0))
 	return 0;
     va_start(argp, format);
@@ -747,12 +756,12 @@ checkStatusReplyOK(redisInfoCallBack info, void *userdata,
 }
 
 int
-checkStreamReplyString(redisInfoCallBack info, void *userdata,
-	redisClusterAsyncContext *acc, redisReply *reply, sds s, const char *format, ...)
+checkStreamReplyString(keysInfoCallBack info, void *userdata,
+	keyClusterAsyncContext *acc, respReply *reply, sds s, const char *format, ...)
 {
     va_list		argp;
 
-    if (reply && reply->type == REDIS_REPLY_STRING && strcmp(s, reply->str) == 0)
+    if (reply && reply->type == RESP_REPLY_STRING && strcmp(s, reply->str) == 0)
 	return 0;
     va_start(argp, format);
     reportReplyError(info, userdata, acc, reply, format, argp);
@@ -761,12 +770,12 @@ checkStreamReplyString(redisInfoCallBack info, void *userdata,
 }
 
 int
-checkArrayReply(redisInfoCallBack info, void *userdata,
-	redisClusterAsyncContext *acc, redisReply *reply, const char *format, ...)
+checkArrayReply(keysInfoCallBack info, void *userdata,
+	keyClusterAsyncContext *acc, respReply *reply, const char *format, ...)
 {
     va_list		argp;
 
-    if (reply && reply->type == REDIS_REPLY_ARRAY)
+    if (reply && reply->type == RESP_REPLY_ARRAY)
 	return 0;
     va_start(argp, format);
     reportReplyError(info, userdata, acc, reply, format, argp);
@@ -775,12 +784,12 @@ checkArrayReply(redisInfoCallBack info, void *userdata,
 }
 
 long long
-checkIntegerReply(redisInfoCallBack info, void *userdata,
-	redisClusterAsyncContext *acc, redisReply *reply, const char *format, ...)
+checkIntegerReply(keysInfoCallBack info, void *userdata,
+	keyClusterAsyncContext *acc, respReply *reply, const char *format, ...)
 {
     va_list		argp;
 
-    if (reply && reply->type == REDIS_REPLY_INTEGER)
+    if (reply && reply->type == RESP_REPLY_INTEGER)
 	return reply->integer;
     va_start(argp, format);
     reportReplyError(info, userdata, acc, reply, format, argp);
@@ -789,12 +798,12 @@ checkIntegerReply(redisInfoCallBack info, void *userdata,
 }
 
 sds
-checkStringReply(redisInfoCallBack info, void *userdata,
-	redisClusterAsyncContext *acc, redisReply *reply, const char *format, ...)
+checkStringReply(keysInfoCallBack info, void *userdata,
+	keyClusterAsyncContext *acc, respReply *reply, const char *format, ...)
 {
     va_list		argp;
 
-    if (reply && reply->type == REDIS_REPLY_STRING)
+    if (reply && reply->type == RESP_REPLY_STRING)
 	return sdsnew(reply->str);
     va_start(argp, format);
     reportReplyError(info, userdata, acc, reply, format, argp);
