@@ -2021,7 +2021,7 @@ pmWebGroupScrape(pmWebGroupSettings *settings, sds id, dict *params, void *arg)
 	    if (strcmp(match, "regex") == 0) {
 		scrape.match = MATCH_REGEX;
 	    } else if (strcmp(match, "exact") == 0)
-		scrape.match = MATCH_GLOB;
+		scrape.match = MATCH_EXACT;
 	    else if (strcmp(match, "glob") != 0) {
 		infofmt(msg, "%s - invalid 'match' parameter value", match);
 		sts = -EINVAL;
