@@ -99,23 +99,23 @@ static time_t	daylimit(time_t);
 static char *
 saroptions(void)
 {
-	char		*flags;
+	static char	*allflags;	/* static for valgrind */
 	int		i;
 
 	/* 
 	** gather all flags for the print-functions
 	*/
-	flags = calloc(1, pricnt+32);
+	allflags = calloc(1, pricnt+32);
 
-	ptrverify(flags, "Malloc failed for %d flags\n", pricnt+32);
+	ptrverify(allflags, "Malloc failed for %d flags\n", pricnt+32);
 
 	for (i=0; i < pricnt; i++)
-		flags[i] = pridef[i].flag;
+		allflags[i] = pridef[i].flag;
 
 	/*
 	** add generic flags
 	*/
-	return strncat(flags, sarflags, pricnt+32 - 1);
+	return strncat(allflags, sarflags, pricnt+32 - 1);
 }
 
 int
@@ -936,13 +936,15 @@ pratopsaruse(char *myname, pmOptions *opts)
 static time_t
 daylimit(time_t timval)
 {
-	struct tm  *tp = localtime(&timval);
+	struct tm  tt, *tp;
+
+	tp = pmLocaltime(&timval, &tt);
 
 	tp->tm_hour = 23;
 	tp->tm_min  = 59;
 	tp->tm_sec  = 59;
 
-	return mktime(tp);
+	return __pmMktime(tp);
 }
 
 /*
@@ -2618,7 +2620,7 @@ topnline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 		return 0;
 	}
 
-	if ( !(supportflags & NETATOP) )
+	if ( !(supportflags & NETATOP || supportflags & NETATOPBPF) )
 	{
 		printf("no per-process network counters available.....\n");
 		return 0;
