@@ -1064,12 +1064,16 @@ ParseHosts(const char *source, int allow)
 	}
 	else
 	    ok = 1;
+	free(hostnames[i]);
+	hostnames[i] = NULL;
     }
     return ok;
 
 error:
-    for (i = 0; i < nhosts; i++)
-	free(hostnames[i]);
+    for (i = 0; i < nhosts; i++) {
+	if (hostnames[i] != NULL)
+	    free(hostnames[i]);
+    }
     return -1;
 }
 
@@ -1542,7 +1546,7 @@ AgentNegotiate(AgentInfo *aPtr)
     int		sts;
     __pmPDU	*ack;
 
-    sts = __pmGetPDU(aPtr->outFd, ANY_SIZE, _creds_timeout, &ack);
+    sts = __pmGetPDU(aPtr->outFd, ANY_SIZE, creds_timeout, &ack);
     if (sts == PDU_CREDS) {
 	if ((sts = DoAgentCreds(aPtr, ack)) < 0) {
 	    fprintf(stderr, "pmcd: version exchange failed "
