@@ -28,7 +28,7 @@ BEGIN {
 
 my $rest_client = mock;
 
-my $queue = Queue->new("org.apache.activemq:type=Broker,brokerName=localhost,destinationType=Queue,destinationName=queuename", $rest_client);
+my $queue = PCP::Queue->new("org.apache.activemq:type=Broker,brokerName=localhost,destinationType=Queue,destinationName=queuename", $rest_client);
 
 when($rest_client)->get("/api/jolokia/read/org.apache.activemq:type=Broker,brokerName=localhost,destinationType=Queue,destinationName=queuename?ignoreErrors=true")->then_return({'value' => { 'DequeueCount' => 99}});
 is($queue->attribute_for('dequeue_count'), 99, "DequeueCount is available as an attribute_for");
