@@ -13,7 +13,7 @@
  * 
  * You should have received a copy of the GNU General Public License along
  * with this program; if not, write to the Free Software Foundation, Inc.,
- * 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+ * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #include <ctype.h>
@@ -103,14 +103,13 @@ refresh(void *dummy)
 #endif
 #endif
 
-#ifdef PCP_DEBUG
-    if (pmDebug & DBG_TRACE_APPL0) {
+    if (pmDebugOptions.appl0) {
 	fprintf(stderr, "Starting sproc ...\n");
 	for (i = 0; i < n_cisco; i++) {
 	    int	j;
 
-	    fprintf(stderr, "cisco[%d] host: %s username: %s passwd: %s prompt: %s intf:",
-		    i, cisco[i].host, cisco[i].username, cisco[i].passwd, cisco[i].prompt);
+	    fprintf(stderr, "cisco[%d] host: %s username: %s prompt: %s intf:",
+		    i, cisco[i].host, cisco[i].username, cisco[i].prompt);
 
 	    for (j = 0; j < n_intf; j++) {
 	        if (intf[j].cp == (cisco+i))
@@ -119,7 +118,6 @@ refresh(void *dummy)
 	    fputc('\n', stderr);
 	}
     }
-#endif
 
     for ( ; ; ) {
 	for (i = 0; i < n_intf; i++) {
@@ -135,25 +133,19 @@ refresh(void *dummy)
 
 	for (i = 0; i < n_cisco; i++) {
 	    if (cisco[i].fout != NULL) {
-#ifdef PCP_DEBUG
-		if (pmDebug & DBG_TRACE_APPL0)
+		if (pmDebugOptions.appl0)
 		    fprintf(stderr, "... %s voluntary disconnect fout=%d\n", cisco[i].host, fileno(cisco[i].fout));
-#endif
 		/* close CISCO telnet session */
-#ifdef PCP_DEBUG
-		if (pmDebug & DBG_TRACE_APPL1) {
+		if (pmDebugOptions.appl1) {
 		    fprintf(stderr, "Send: exit\n");
 		}
-#endif
 		fprintf(cisco[i].fout, "exit\n");
 		fclose(cisco[i].fout);
 		cisco[i].fout = NULL;
 	    }
 	    if (cisco[i].fin != NULL) {
-#ifdef PCP_DEBUG
-		if (pmDebug & DBG_TRACE_APPL0)
+		if (pmDebugOptions.appl0)
 		    fprintf(stderr, "... %s close fin=%d\n", cisco[i].host, fileno(cisco[i].fin));
-#endif
 		fclose(cisco[i].fin);
 		cisco[i].fin = NULL;
 	    }
@@ -166,8 +158,6 @@ refresh(void *dummy)
 static int
 cisco_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *avp)
 {
-    __pmID_int	*idp = (__pmID_int *)&(mdesc->m_desc.pmid);
-
 #ifndef HAVE_SPROC
     /* Check is refresh thread is still with us */
     int err;
@@ -180,7 +170,7 @@ cisco_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *avp)
     if (!intf[inst].fetched)
 	return PM_ERR_AGAIN;
 
-    switch (idp->item) {
+    switch (pmID_item(mdesc->m_desc.pmid)) {
 
 	case 1:		/* bytes_in */
 		if (intf[inst].bytes_in == -1) return 0;
