@@ -24,10 +24,10 @@
 #include <sys/utsname.h>
 #include <mach/mach.h>
 #include "pmapi.h"
-#include "impl.h"
 #include "pmda.h"
 #include "domain.h"
 
+#include "darwin.h"
 #include "disk.h"
 #include "network.h"
 
@@ -87,6 +87,9 @@ int			mach_nfs_error = 0;
 struct nfsstats		mach_nfs = { 0 };
 extern int refresh_nfs(struct nfsstats *);
 
+char			hw_model[MODEL_SIZE];
+extern int refresh_hinv(void);
+
 /*
  * Metric Instance Domains (statically initialized ones only)
  */
@@ -117,7 +120,7 @@ enum {
     DISK_INDOM,			/* 2 - set of all disk devices */
     CPU_INDOM,			/* 3 - set of all processors */
     NETWORK_INDOM,		/* 4 - set of all network interfaces */
-    NFS3_INDOM,			/* 6 - nfs v3 operations */
+    NFS3_INDOM,			/* 5 - nfs v3 operations */
     NUM_INDOMS			/* total number of instance domains */
 };
 
@@ -159,6 +162,10 @@ static pmdaMetric metrictab[] = {
   { &mach_hertz,
     { PMDA_PMID(CLUSTER_INIT,1), PM_TYPE_U32, PM_INDOM_NULL,
       PM_SEM_DISCRETE, PMDA_PMUNITS(0,-1,1,0,PM_TIME_SEC,PM_COUNT_ONE) }, },
+/* hinv.machine */
+  { &hw_model,
+    { PMDA_PMID(CLUSTER_INIT,2), PM_TYPE_STRING, PM_INDOM_NULL,
+      PM_SEM_DISCRETE, PMDA_PMUNITS(0,0,0,0,0,0) }, },
 
 /* hinv.physmem */
   { NULL, 
@@ -244,6 +251,10 @@ static pmdaMetric metrictab[] = {
   { NULL,
     { PMDA_PMID(CLUSTER_VMSTAT,22), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_INSTANT, PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) }, },
+/* mem.util.used */
+  { NULL,
+    { PMDA_PMID(CLUSTER_VMSTAT,23), PM_TYPE_U64, PM_INDOM_NULL,
+      PM_SEM_INSTANT, PMDA_PMUNITS(1,0,0,PM_SPACE_KBYTE,0,0) }, },
 
 /* kernel.uname.release */
   { mach_uname.release,
@@ -539,119 +550,119 @@ static pmdaMetric metrictab[] = {
 
 /* nfs3.client.calls */
   { NULL,
-    { PMDA_PMID(CLUSTER_NFS,94), PM_TYPE_32, PM_INDOM_NULL,
+    { PMDA_PMID(CLUSTER_NFS,94), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* nfs3.client.reqs */
   { NULL,
-    { PMDA_PMID(CLUSTER_NFS,95), PM_TYPE_32, NFS3_INDOM,
+    { PMDA_PMID(CLUSTER_NFS,95), PM_TYPE_U64, NFS3_INDOM,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* nfs3.server.calls */
   { NULL,
-    { PMDA_PMID(CLUSTER_NFS,96), PM_TYPE_32, PM_INDOM_NULL,
+    { PMDA_PMID(CLUSTER_NFS,96), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* nfs3.server.reqs */
   { NULL,
-    { PMDA_PMID(CLUSTER_NFS,97), PM_TYPE_32, NFS3_INDOM,
+    { PMDA_PMID(CLUSTER_NFS,97), PM_TYPE_U64, NFS3_INDOM,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.rpccnt */
-  { &mach_nfs.rpcrequests,
-    { PMDA_PMID(CLUSTER_NFS,98), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.rpcrequests,
+    { PMDA_PMID(CLUSTER_NFS,98), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.rpcretrans */
-  { &mach_nfs.rpcretries,
-    { PMDA_PMID(CLUSTER_NFS,99), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.rpcretries,
+    { PMDA_PMID(CLUSTER_NFS,99), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.rpctimeouts */
-  { &mach_nfs.rpctimeouts,
-    { PMDA_PMID(CLUSTER_NFS,100), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.rpctimeouts,
+    { PMDA_PMID(CLUSTER_NFS,100), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.rpcinvalid */
-  { &mach_nfs.rpcinvalid,
-    { PMDA_PMID(CLUSTER_NFS,101), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.rpcinvalid,
+    { PMDA_PMID(CLUSTER_NFS,101), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.rpcunexpected */
-  { &mach_nfs.rpcunexpected,
-    { PMDA_PMID(CLUSTER_NFS,102), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.rpcunexpected,
+    { PMDA_PMID(CLUSTER_NFS,102), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.attrcache.hits */
-  { &mach_nfs.attrcache_hits,
-    { PMDA_PMID(CLUSTER_NFS,103), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.attrcache_hits,
+    { PMDA_PMID(CLUSTER_NFS,103), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.attrcache.misses */
-  { &mach_nfs.attrcache_misses,
-    { PMDA_PMID(CLUSTER_NFS,104), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.attrcache_misses,
+    { PMDA_PMID(CLUSTER_NFS,104), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.lookupcache.hits */
-  { &mach_nfs.lookupcache_hits,
-    { PMDA_PMID(CLUSTER_NFS,105), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.lookupcache_hits,
+    { PMDA_PMID(CLUSTER_NFS,105), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.lookupcache.misses */
-  { &mach_nfs.lookupcache_misses,
-    { PMDA_PMID(CLUSTER_NFS,106), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.lookupcache_misses,
+    { PMDA_PMID(CLUSTER_NFS,106), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.biocache.read.hits */
-  { &mach_nfs.read_bios,
-    { PMDA_PMID(CLUSTER_NFS,107), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.read_bios,
+    { PMDA_PMID(CLUSTER_NFS,107), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.biocache.read.misses */
-  { &mach_nfs.biocache_reads,
-    { PMDA_PMID(CLUSTER_NFS,108), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.biocache_reads,
+    { PMDA_PMID(CLUSTER_NFS,108), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.biocache.write.hits */
-  { &mach_nfs.write_bios,
-    { PMDA_PMID(CLUSTER_NFS,109), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.write_bios,
+    { PMDA_PMID(CLUSTER_NFS,109), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.biocache.write.misses */
-  { &mach_nfs.biocache_writes,
-    { PMDA_PMID(CLUSTER_NFS,110), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.biocache_writes,
+    { PMDA_PMID(CLUSTER_NFS,110), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.biocache.readlink.hits */
-  { &mach_nfs.readlink_bios,
-    { PMDA_PMID(CLUSTER_NFS,111), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.readlink_bios,
+    { PMDA_PMID(CLUSTER_NFS,111), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.biocache.readlink.misses */
-  { &mach_nfs.biocache_readlinks,
-    { PMDA_PMID(CLUSTER_NFS,112), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.biocache_readlinks,
+    { PMDA_PMID(CLUSTER_NFS,112), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.biocache.readdir.hits */
-  { &mach_nfs.readdir_bios,
-    { PMDA_PMID(CLUSTER_NFS,113), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.readdir_bios,
+    { PMDA_PMID(CLUSTER_NFS,113), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.biocache.readdir.misses */
-  { &mach_nfs.biocache_readdirs,
-    { PMDA_PMID(CLUSTER_NFS,114), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.biocache_readdirs,
+    { PMDA_PMID(CLUSTER_NFS,114), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.direofcache.hits */
-  { &mach_nfs.direofcache_hits,
-    { PMDA_PMID(CLUSTER_NFS,115), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.direofcache_hits,
+    { PMDA_PMID(CLUSTER_NFS,115), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.client.direofcache.misses */
-  { &mach_nfs.direofcache_misses,
-    { PMDA_PMID(CLUSTER_NFS,116), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.client.direofcache_misses,
+    { PMDA_PMID(CLUSTER_NFS,116), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.server.retfailed */
-  { &mach_nfs.srvrpc_errs,
-    { PMDA_PMID(CLUSTER_NFS,117), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.server.srvrpc_errs,
+    { PMDA_PMID(CLUSTER_NFS,117), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.server.faults */
-  { &mach_nfs.srvrpc_errs,
-    { PMDA_PMID(CLUSTER_NFS,118), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.server.srvrpc_errs,
+    { PMDA_PMID(CLUSTER_NFS,118), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.server.cache.inprog */
-  { &mach_nfs.srvcache_inproghits,
-    { PMDA_PMID(CLUSTER_NFS,119), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.server.srvcache_inproghits,
+    { PMDA_PMID(CLUSTER_NFS,119), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.server.cache.nonidem */
-  { &mach_nfs.srvcache_nonidemdonehits,
-    { PMDA_PMID(CLUSTER_NFS,120), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.server.srvcache_nonidemdonehits,
+    { PMDA_PMID(CLUSTER_NFS,120), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.server.cache.idem */
-  { &mach_nfs.srvcache_idemdonehits,
-    { PMDA_PMID(CLUSTER_NFS,121), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.server.srvcache_idemdonehits,
+    { PMDA_PMID(CLUSTER_NFS,121), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.server.cache.misses */
-  { &mach_nfs.srvcache_misses,
-    { PMDA_PMID(CLUSTER_NFS,122), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.server.srvcache_misses,
+    { PMDA_PMID(CLUSTER_NFS,122), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.server.nqnfs.leases -- deprecated */
   { NULL,
@@ -666,16 +677,16 @@ static pmdaMetric metrictab[] = {
     { PMDA_PMID(CLUSTER_NFS,125), PM_TYPE_NOSUPPORT, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 /* rpc.server.vopwrites */
-  { &mach_nfs.srvvop_writes,
-    { PMDA_PMID(CLUSTER_NFS,126), PM_TYPE_32, PM_INDOM_NULL,
+  { &mach_nfs.server.srvvop_writes,
+    { PMDA_PMID(CLUSTER_NFS,126), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
-/* rpc.server.pageins */
-  { &mach_nfs.pageins,
-    { PMDA_PMID(CLUSTER_NFS,127), PM_TYPE_32, PM_INDOM_NULL,
+/* rpc.client.pageins */
+  { &mach_nfs.client.pageins,
+    { PMDA_PMID(CLUSTER_NFS,127), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
-/* rpc.server.pageouts */
-  { &mach_nfs.pageouts,
-    { PMDA_PMID(CLUSTER_NFS,128), PM_TYPE_32, PM_INDOM_NULL,
+/* rpc.client.pageouts */
+  { &mach_nfs.client.pageouts,
+    { PMDA_PMID(CLUSTER_NFS,128), PM_TYPE_U64, PM_INDOM_NULL,
       PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) }, },
 
 /* filesys.maxfiles */
@@ -793,6 +804,9 @@ fetch_vmstat(unsigned int item, unsigned int inst, pmAtomValue *atom)
     case 22: /* mem.util.free */
 	atom->ull = page_count_to_kb(mach_vmstat.free_count);
 	return 1;
+    case 23: /* mem.util.used */
+	atom->ull = page_count_to_kb(mach_vmstat.wire_count+mach_vmstat.active_count+mach_vmstat.inactive_count);
+	return 1;
     }
     return PM_ERR_PMID;
 }
@@ -806,7 +820,7 @@ fetch_uname(unsigned int item, pmAtomValue *atom)
 	return mach_uname_error;
     switch (item) {
     case 28: /* pmda.uname */
-	snprintf(mach_uname_all, sizeof(mach_uname_all), "%s %s %s %s %s",
+	pmsprintf(mach_uname_all, sizeof(mach_uname_all), "%s %s %s %s %s",
 		mach_uname.sysname, mach_uname.nodename,
 		mach_uname.release, mach_uname.version,
 		mach_uname.machine);
@@ -1080,26 +1094,26 @@ fetch_network(unsigned int item, unsigned int inst, pmAtomValue *atom)
 static inline int
 fetch_nfs(unsigned int item, unsigned int inst, pmAtomValue *atom)
 {
-    if (mach_net_error)
-	return mach_net_error;
+    if (mach_nfs_error)
+	return mach_nfs_error;
     switch (item) {
     case 94: /* nfs3.client.calls */
-	for (atom->l = 0, inst = 0; inst < NFS3_RPC_COUNT; inst++)
-	    atom->l += mach_nfs.rpccnt[inst];
+	for (atom->ull = 0, inst = 0; inst < NFS3_RPC_COUNT; inst++)
+	    atom->ull += mach_nfs.client.rpccntv3[inst];
 	return 1;
     case 95: /* nfs3.client.reqs */
 	if (inst < 0 || inst >= NFS3_RPC_COUNT)
 	    return PM_ERR_INST;
-	atom->l = mach_nfs.rpccnt[inst];
+	atom->ull = mach_nfs.client.rpccntv3[inst];
 	return 1;
     case 96: /* nfs3.server.calls */
-	for (atom->l = 0, inst = 0; inst < NFS3_RPC_COUNT; inst++)
-	    atom->l += mach_nfs.srvrpccnt[inst];
+	for (atom->ull = 0, inst = 0; inst < NFS3_RPC_COUNT; inst++)
+	    atom->ull += mach_nfs.server.srvrpccntv3[inst];
 	return 1;
     case 97: /* nfs3.server.reqs */
 	if (inst < 0 || inst >= NFS3_RPC_COUNT)
 	    return PM_ERR_INST;
-	atom->l = mach_nfs.srvrpccnt[inst];
+	atom->ull = mach_nfs.server.srvrpccntv3[inst];
 	return 1;
     case 123:	/* rpc.server.nqnfs.leases    -- deprecated */
     case 124:	/* rpc.server.nqnfs.maxleases -- deprecated */
@@ -1113,7 +1127,7 @@ fetch_nfs(unsigned int item, unsigned int inst, pmAtomValue *atom)
 static int
 darwin_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 {
-    __pmID_int	*idp = (__pmID_int *)&(mdesc->m_desc.pmid);
+    unsigned int	item;
 
     if (mdesc->m_user) {
 	/*   
@@ -1139,27 +1153,27 @@ darwin_fetchCallBack(pmdaMetric *mdesc, unsigned int inst, pmAtomValue *atom)
 	return 1;
     }
 
-    switch (idp->cluster) {
-    case CLUSTER_LOADAVG:	return fetch_loadavg(idp->item, inst, atom);
-    case CLUSTER_CPULOAD:	return fetch_cpuload(idp->item, atom);
-    case CLUSTER_VMSTAT:	return fetch_vmstat(idp->item, inst, atom);
-    case CLUSTER_KERNEL_UNAME:	return fetch_uname(idp->item, atom);
-    case CLUSTER_FILESYS:	return fetch_filesys(idp->item, inst, atom);
-    case CLUSTER_DISK:		return fetch_disk(idp->item, inst, atom);
-    case CLUSTER_CPU:		return fetch_cpu(idp->item, inst, atom);
-    case CLUSTER_NETWORK:	return fetch_network(idp->item, inst, atom);
-    case CLUSTER_NFS:		return fetch_nfs(idp->item, inst, atom);
+    item = pmID_item(mdesc->m_desc.pmid);
+    switch (pmID_cluster(mdesc->m_desc.pmid)) {
+    case CLUSTER_LOADAVG:	return fetch_loadavg(item, inst, atom);
+    case CLUSTER_CPULOAD:	return fetch_cpuload(item, atom);
+    case CLUSTER_VMSTAT:	return fetch_vmstat(item, inst, atom);
+    case CLUSTER_KERNEL_UNAME:	return fetch_uname(item, atom);
+    case CLUSTER_FILESYS:	return fetch_filesys(item, inst, atom);
+    case CLUSTER_DISK:		return fetch_disk(item, inst, atom);
+    case CLUSTER_CPU:		return fetch_cpu(item, inst, atom);
+    case CLUSTER_NETWORK:	return fetch_network(item, inst, atom);
+    case CLUSTER_NFS:		return fetch_nfs(item, inst, atom);
     }
     return 0;
 }
 
 static int
-darwin_instance(pmInDom indom, int inst, char *name, __pmInResult **result, pmdaExt *pmda)
+darwin_instance(pmInDom indom, int inst, char *name, pmInResult **result, pmdaExt *pmda)
 {
-    __pmInDom_int	*indomp = (__pmInDom_int *)&indom;
     int			need_refresh[NUM_CLUSTERS] = { 0 };
 
-    switch (indomp->serial) {
+    switch (pmInDom_serial(indom)) {
     case FILESYS_INDOM: need_refresh[CLUSTER_FILESYS]++; break;
     case DISK_INDOM:	need_refresh[CLUSTER_DISK]++; break;
     case CPU_INDOM:	need_refresh[CLUSTER_CPU]++; break;
@@ -1175,9 +1189,9 @@ darwin_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
     int	i, need_refresh[NUM_CLUSTERS] = { 0 };
 
     for (i = 0; i < numpmid; i++) {
-	__pmID_int *idp = (__pmID_int *)&(pmidlist[i]);
-	if (idp->cluster >= 0 && idp->cluster < NUM_CLUSTERS)
-	    need_refresh[idp->cluster]++;
+	unsigned int	cluster = pmID_cluster(pmidlist[i]);
+	if (cluster >= 0 && cluster < NUM_CLUSTERS)
+	    need_refresh[cluster]++;
     }
     darwin_refresh(need_refresh);
     return pmdaFetch(numpmid, pmidlist, resp, pmda);
@@ -1186,14 +1200,16 @@ darwin_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 void 
 darwin_init(pmdaInterface *dp)
 {
+    int		sts;
+
     if (_isDSO) {
-	int sep = __pmPathSeparator();
+	int sep = pmPathSeparator();
 	char helppath[MAXPATHLEN];
-	sprintf(helppath, "%s%c" "darwin" "%c" "help",
+	pmsprintf(helppath, MAXPATHLEN, "%s%c" "darwin" "%c" "help",
 		pmGetConfig("PCP_PMDAS_DIR"), sep, sep);
 	pmdaDSO(dp, PMDA_INTERFACE_3, "darwin DSO", helppath);
     } else {
-	__pmSetProcessIdentity(username);
+	pmSetProcessIdentity(username);
     }
 
     if (dp->status != 0)
@@ -1203,25 +1219,24 @@ darwin_init(pmdaInterface *dp)
     dp->version.two.fetch = darwin_fetch;
     pmdaSetFetchCallBack(dp, darwin_fetchCallBack);
 
+    pmdaSetFlags(dp, PMDA_EXT_FLAG_HASHED);
     pmdaInit(dp, indomtab, sizeof(indomtab)/sizeof(indomtab[0]),
 		metrictab, sizeof(metrictab)/sizeof(metrictab[0]));
 
-    /* We should have a direct metric table lookup */
-    if (!dp->version.two.ext->e_direct)
-	__pmNotifyErr(LOG_INFO, "pmdadarwin: direct metric lookup disabled\n");
-
     mach_host = mach_host_self();
     host_page_size(mach_host, &mach_page_size);
     mach_page_shift = ffs(mach_page_size) - 1;
     if (refresh_hertz(&mach_hertz) != 0)
 	mach_hertz = 100;
+    if ((sts = refresh_hinv()) != 0)
+	fprintf(stderr, "darwin_init: refresh_hinv failed: %s\n", pmErrStr(sts));
     init_network();
 }
 
 static void
 usage(void)
 {
-    fprintf(stderr, "Usage: %s [options]\n\n", pmProgname);
+    fprintf(stderr, "Usage: %s [options]\n\n", pmGetProgname());
     fputs("Options:\n"
 "  -d domain    use domain (numeric) for metrics domain of PMDA\n"
 "  -l logfile   write log into logfile rather than using default log name\n"
@@ -1229,7 +1244,8 @@ usage(void)
 "\nExactly one of the following options may appear:\n"
 "  -i port      expect PMCD to connect on given inet port (number or name)\n"
 "  -p           expect PMCD to supply stdin/stdout (pipe)\n"
-"  -u socket    expect PMCD to connect on given unix domain socket\n",
+"  -u socket    expect PMCD to connect on given unix domain socket\n"
+"  -6 port      expect PMCD to connect on given ipv6 port (number or name)\n",
 	  stderr);		
     exit(1);
 }
@@ -1237,20 +1253,20 @@ usage(void)
 int
 main(int argc, char **argv)
 {
-    int			c, sep = __pmPathSeparator();
+    int			c, sep = pmPathSeparator();
     int			errflag = 0;
     char		helppath[MAXPATHLEN];
 
     _isDSO = 0;
-    __pmSetProgname(argv[0]);
-    __pmGetUsername(&username);
+    pmSetProgname(argv[0]);
+    pmGetUsername(&username);
 
-    sprintf(helppath, "%s%c" "darwin" "%c" "help",
+    pmsprintf(helppath, MAXPATHLEN, "%s%c" "darwin" "%c" "help",
 		pmGetConfig("PCP_PMDAS_DIR"), sep, sep);
-    pmdaDaemon(&dispatch, PMDA_INTERFACE_3, pmProgname, DARWIN, "darwin.log",
+    pmdaDaemon(&dispatch, PMDA_INTERFACE_3, pmGetProgname(), DARWIN, "darwin.log",
 		helppath);
 
-    while ((c = pmdaGetOpt(argc, argv, "D:d:i:l:pu:U:?", &dispatch, &errflag)) != EOF) {
+    while ((c = pmdaGetOpt(argc, argv, "D:d:i:l:pu:U:6:?", &dispatch, &errflag)) != EOF) {
 	switch(c) {
 	case 'U':
 	    username = optarg;
