@@ -23,8 +23,11 @@ typedef struct filesys {
     int		  id;
     unsigned int  flags;
     char	  *device;
+    char	  *dm_device;	/* corresponding device mapper device */
     char	  *path;
+    char	  *type;
     char	  *options;
+    char	  *uuid;
     struct statfs stats;
 } filesys_t;
 
