@@ -47,6 +47,8 @@ static struct {
     { "MmapCopy",	&moff.MmapCopy },
     { "SwapTotal",	&moff.SwapTotal },
     { "SwapFree",	&moff.SwapFree },
+    { "Zswap",		&moff.Zswap },
+    { "Zswapped",	&moff.Zswapped },
     { "Dirty",		&moff.Dirty },
     { "Writeback",	&moff.Writeback },
     { "AnonPages",	&moff.AnonPages },
@@ -56,6 +58,7 @@ static struct {
     { "SReclaimable",	&moff.SlabReclaimable },
     { "SUnreclaim",	&moff.SlabUnreclaimable },
     { "KernelStack",	&moff.KernelStack },
+    { "ShadowCallStack", &moff.ShadowCallStack },
     { "PageTables",	&moff.PageTables },
     { "Quicklists",	&moff.Quicklists },
     { "NFS_Unstable",	&moff.NFS_Unstable },
@@ -66,8 +69,16 @@ static struct {
     { "VmallocTotal",	&moff.VmallocTotal },
     { "VmallocUsed",	&moff.VmallocUsed },
     { "VmallocChunk",	&moff.VmallocChunk },
+    { "Percpu",		&moff.Percpu },
     { "HardwareCorrupted", &moff.HardwareCorrupted },
     { "AnonHugePages",	&moff.AnonHugePages },
+    { "ShmemHugePages",	&moff.ShmemHugePages },
+    { "ShmemPmdMapped",	&moff.ShmemPmdMapped },
+    { "FileHugePages",	&moff.FileHugePages },
+    { "FilePmdMapped",	&moff.FilePmdMapped },
+    { "CmaTotal",	&moff.CmaTotal },
+    { "CmaFree",	&moff.CmaFree },
+    { "Unaccepted",	&moff.Unaccepted },
     /* vendor kernel patches, some outdated now */
     { "MemShared",	&moff.MemShared },
     { "ReverseMaps",	&moff.ReverseMaps },
@@ -125,8 +136,7 @@ refresh_proc_meminfo(proc_meminfo_t *proc_meminfo)
      * MemAvailable is only in 3.x or later kernels but we can calculate it
      * using other values, similar to upstream kernel commit 34e431b0ae.
      */
-    if (!MEMINFO_VALID_VALUE(proc_meminfo->MemAvailable) ||
-	(linux_test_mode & LINUX_TEST_MEMINFO)) {
+    if (!MEMINFO_VALID_VALUE(proc_meminfo->MemAvailable)) {
 	if (MEMINFO_VALID_VALUE(proc_meminfo->MemTotal) &&
 	    MEMINFO_VALID_VALUE(proc_meminfo->MemFree) &&
 	    MEMINFO_VALID_VALUE(proc_meminfo->Active_file) &&
