@@ -247,6 +247,13 @@ enum {
     _PM_NETSTAT_MPTCPEXT_MPCAPABLEENDPATTEMPT,
     _PM_NETSTAT_MPTCPEXT_DSSCORRUPTIONFALLBACK,
     _PM_NETSTAT_MPTCPEXT_DSSCORRUPTIONRESET,
+    _PM_NETSTAT_MPTCPEXT_MPCAPABLESYNTXDROP,
+    _PM_NETSTAT_MPTCPEXT_MPCAPABLESYNTXDISABLED,
+    _PM_NETSTAT_MPTCPEXT_MPJOINSYNTX,
+    _PM_NETSTAT_MPTCPEXT_MPJOINSYNTXCREATSKERR,
+    _PM_NETSTAT_MPTCPEXT_MPJOINSYNTXBINDERR,
+    _PM_NETSTAT_MPTCPEXT_MPJOINSYNTXCONNECTERR,
+    _PM_NETSTAT_MPTCPEXT_BLACKHOLE,
 
     _PM_NETSTAT_MPTCPEXT_NFIELDS /* must be last */
 };
