@@ -121,7 +121,8 @@ free_parameters(int nparams, char **paramtab)
 {
     while (--nparams >= 0)
 	free(paramtab[nparams]);
-    free(paramtab);
+    if (paramtab != NULL)
+	free(paramtab);
 }
 
 /* replace pipe command string wildcards with user-supplied parameters */
@@ -193,22 +194,7 @@ setup_cmdline(pipe_client *pc, pipe_command *cmd, char *params)
 			paramlen++;
 		    }
 		    len = strlen(paramtab[j]);
-#ifdef __GNUC__
-#if __GNUC__ >= 8
-		    /*
-		     * gcc 8 on Fedora 30 emits warnings for this strncat()
-		     * calls ... but it is safe
-		     */
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wstringop-overflow"
-#endif
-#endif
-		    strncat(q, paramtab[j], len+1);
-#ifdef __GNUC__
-#if __GNUC__ >= 8
-#pragma GCC diagnostic pop
-#endif
-#endif
+		    strncat(q, paramtab[j], MAXPATHLEN - (q - buffer) -1);
 		    paramlen += len;
 		}
 		assert(paramlen == total);
@@ -870,7 +856,11 @@ event_acl(pmInDom aclops)
 	if (strcmp(pa->identifier, "*") == 0)
 	    continue;
 	pmdaCacheStore(aclops, PMDA_CACHE_ADD, pa->identifier, NULL);
-	pmdaCacheLookupName(aclops, pa->identifier, &pa->operation, NULL);
+	if ((sts = pmdaCacheLookupName(aclops, pa->identifier, &pa->operation, NULL)) < 0) {
+	    pmNotifyErr(LOG_ERR, "event_acl: pmdaCacheLookupName(...,%s,...): %s",
+		    pa->identifier, pmErrStr(sts));
+	    exit(1);
+	}
 
 	if (pa->user)
 	    nusers++;
@@ -943,6 +933,7 @@ event_indom(pmInDom pipe_indom)
 {
     pipe_command	*pc;
     int			i;
+    int			sts;
 
     /* initialize the instance domain cache */
     pmdaCacheOp(pipe_indom, PMDA_CACHE_LOAD);
@@ -951,7 +942,11 @@ event_indom(pmInDom pipe_indom)
     for (i = 0; i < cmdtab_size; i++) {
 	pc = &cmdtab[i];
 	pmdaCacheStore(pipe_indom, PMDA_CACHE_ADD, pc->identifier, pc);
-	pmdaCacheLookupName(pipe_indom, pc->identifier, &pc->inst, NULL);
+	if ((sts = pmdaCacheLookupName(pipe_indom, pc->identifier, &pc->inst, NULL)) < 0) {
+	    pmNotifyErr(LOG_ERR, "event_indom: pmdaCacheLookupName(...,%s,...): %s",
+		    pc->identifier, pmErrStr(sts));
+	    exit(1);
+	}
 
 	if (pmDebugOptions.appl0)
             pmNotifyErr(LOG_DEBUG, "event_indom: added %s[%d]", pc->identifier, pc->inst);
