@@ -21,7 +21,7 @@
  *
  * Parallel program to demonstrate use of the PCP trace performance metrics
  * domain agent (PMDA(3)).  This agent needs to be installed before metrics
- * can be made available via the performance metrics namespace (pmns(5)),
+ * can be made available via the performance metrics namespace (PMNS(5)),
  * and the Performance Metrics Collector Daemon (pmcd(1)).
  *
  * Once this program is running, the trace PMDA metrics & instances can be
@@ -55,13 +55,23 @@ int
 main(int argc, char **argv)
 {
     int		i;
+    int		sts;
     pthread_t p[3];
 
     prog = argv[0];
 
-    pthread_create(p,   NULL, pio_sucker, NULL);
-    pthread_create(p+1, NULL, pcpu_sucker, NULL);
-    pthread_create(p+2, NULL, ptime_sucker, NULL);
+    if ((sts = pthread_create(p,   NULL, pio_sucker, NULL)) != 0) {
+	fprintf(stderr, "%s: pio_sucker pthread_create failed: %s\n", prog, strerror(sts));
+	exit(1);
+    }
+    if ((sts = pthread_create(p+1,   NULL, pcpu_sucker, NULL)) != 0) {
+	fprintf(stderr, "%s: pcpu_sucker pthread_create failed: %s\n", prog, strerror(sts));
+	exit(1);
+    }
+    if ((sts = pthread_create(p+2,   NULL, ptime_sucker, NULL)) != 0) {
+	fprintf(stderr, "%s: ptime_sucker pthread_create failed: %s\n", prog, strerror(sts));
+	exit(1);
+    }
 
     for (i=0; i < 3; i++) {
 	wait(NULL);
