@@ -2,7 +2,7 @@
  * Copyright (c) 2008-2010 Aconex.  All Rights Reserved.
  * Copyright (c) 2004 Silicon Graphics, Inc.  All Rights Reserved.
  * Parts of this file contributed by Ken McDonell
- * (kenj At internode DoT on DoT net)
+ * (kenj At kenj DoT id DoT au)
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -50,17 +50,15 @@ windows_instance_refresh(pmInDom indom)
 int
 windows_lookup_instance(char *path, pdh_metric_t *mp)
 {
-    __pmInDom_int	*ip;
     static void		*seen = (void *)0xfeedbabe;
     void		*sp;
-    char		*p, *q, *name;
+    char		*p, *q, *name = NULL;
     int			sts, ok = 0;
 
     if (mp->desc.indom == PM_INDOM_NULL)
 	return PM_IN_NULL;
 
-    ip = (__pmInDom_int *)&mp->desc.indom;
-    switch (ip->serial) {
+    switch (pmInDom_serial(mp->desc.indom)) {
 	/*
 	 * Examples:
 	 * \\WINBUILD\PhysicalDisk(0 C:)\Disk Reads/sec
@@ -90,9 +88,9 @@ windows_lookup_instance(char *path, pdh_metric_t *mp)
 			    ok = 1;
 			}
 			else {
-			    __pmNotifyErr(LOG_ERR, "windows_check_instance: "
+			    pmNotifyErr(LOG_ERR, "windows_check_instance: "
 					"Error: DISK_INDOM malloc[%d] failed "
-					"path=%s\n", q - p + 1, path);
+					"path=%s\n", (int)(q - p + 1), path);
 			    return -1;
 			}
 			/*
@@ -115,9 +113,10 @@ windows_lookup_instance(char *path, pdh_metric_t *mp)
 	     * entries like "17" and "17#1" are not useable anyway)
 	     */
 	    if (!ok) {
-		if (pmDebug & DBG_TRACE_LIBPMDA)
-		    __pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
+		if (pmDebugOptions.libpmda)
+		    pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
 				"unrecognized disk instance: %s\n", path);
+		free(name);
 		return -1;
 	    }
 	    break;
@@ -141,7 +140,7 @@ windows_lookup_instance(char *path, pdh_metric_t *mp)
 		int inst = atoi(p);
 		name = (char *)malloc(8);	// "cpuNNNN"
 		if (name != NULL)
-		    sprintf(name, "cpu%d", inst);
+		    pmsprintf(name, 8, "cpu%d", inst);
 		ok = 1;
 	    }
 	    /*
@@ -149,8 +148,9 @@ windows_lookup_instance(char *path, pdh_metric_t *mp)
 	     * don't know what to do with this one!
 	     */
 	    if (!ok) {
-		__pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
+		pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
 				"unrecognized cpu instance: %s\n", path);
+		free(name);
 		return -1;
 	    }
 	    break;
@@ -172,9 +172,9 @@ windows_lookup_instance(char *path, pdh_metric_t *mp)
 			ok = 1;
 		    }
 		    else {
-			__pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
+			pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
 					"malloc[%d] failed for NETIF_INDOM "
-					"path=%s\n", q - p + 1, path);
+					"path=%s\n", (int)(q - p + 1), path);
 			return -1;
 		    }
 		    /*
@@ -192,8 +192,9 @@ windows_lookup_instance(char *path, pdh_metric_t *mp)
 	     * don't know what to do with this one!
 	     */
 	    if (!ok) {
-		__pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
+		pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
 			"unrecognized network interface instance: %s\n", path);
+		free(name);
 		return -1;
 	    }
 	    break;
@@ -226,9 +227,9 @@ windows_lookup_instance(char *path, pdh_metric_t *mp)
 			ok = 1;
 		    }
 		    else {
-			__pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
+			pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
 				"malloc[%d] failed for LDISK_INDOM path=%s\n",
-				q - p + 1, path);
+				(int)(q - p + 1), path);
 			return -1;
 		    }
 		}
@@ -238,8 +239,9 @@ windows_lookup_instance(char *path, pdh_metric_t *mp)
 	     * don't know what to do with this one!
 	     */
 	    if (!ok) {
-		__pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
+		pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
 			"unrecognized logical disk instance: %s\n", path);
+		free(name);
 		return -1;
 	    }
 	    break;
@@ -275,9 +277,9 @@ windows_lookup_instance(char *path, pdh_metric_t *mp)
 			ok = 1;
 		    }
 		    else {
-			__pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
+			pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
 				"malloc[%d] failed, SQL_INDOM path=%s\n",
-				q - p + 1, path);
+				(int)(q - p + 1), path);
 			return -1;
 		    }
 
@@ -286,7 +288,7 @@ windows_lookup_instance(char *path, pdh_metric_t *mp)
 		     * not unique up to the first space by any means.  So,
 		     * replace ' 's to play by the PCP instance name rules.
 		     */
-		    if (ip->serial == SQL_USER_INDOM) {
+		    if (pmInDom_serial(mp->desc.indom) == SQL_USER_INDOM) {
 			for (p = name; *p; p++)
 			    if (*p == ' ') *p = '_';
 		    }
@@ -297,8 +299,9 @@ windows_lookup_instance(char *path, pdh_metric_t *mp)
 	     * don't know what to do with this one!
 	     */
 	    if (!ok) {
-		__pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
+		pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
 				"unrecognized SQLServer instance: %s\n", path);
+		free(name);
 		return -1;
 	    }
 	    break;
@@ -334,9 +337,9 @@ windows_lookup_instance(char *path, pdh_metric_t *mp)
 			ok = 1;
 		    }
 		    else {
-			__pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
+			pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
 				"malloc[%d] failed, process/thread path=%s\n",
-				q - p + 1, path);
+				(int)(q - p + 1), path);
 			return -1;
 		    }
 		}
@@ -346,14 +349,15 @@ windows_lookup_instance(char *path, pdh_metric_t *mp)
 	     * don't know what to do with this one!
 	     */
 	    if (!ok) {
-		__pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
+		pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
 				"unrecognized process/thread name: %s\n", path);
+		free(name);
 		return -1;
 	    }
 	    break;
 
 	default:
-	    __pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
+	    pmNotifyErr(LOG_ERR, "windows_check_instance: Error: "
 				   "pmInDom %s is unknown for metric %s\n",
 			pmInDomStr(mp->desc.indom), pmIDStr(mp->desc.pmid));
 	    return -1;
