@@ -51,6 +51,10 @@ char	*clientid;
 
 int	runfromcontrol;
 
+char	*format;			/* for -o|--format */
+char	*configfile;
+int	rule_lineno;
+
 static FILE *logfp;
 static char logfile[MAXPATHLEN];
 static char perffile[MAXPATHLEN];	/* /var/tmp/<pid> file name */
@@ -109,6 +113,7 @@ static pmLongOptions longopts[] = {
     PMAPI_OPTIONS_HEADER("Reporting options"),
     { "buffer", 0, 'b', 0, "one line buffered output stream, stdout on stderr" },
     { "timestamp", 0, 'e', 0, "force timestamps to be reported with -V, -v or -W" },
+    { "format", 1, 'o', "FORMAT", "output format for rule actions in archive mode" },
     { "quiet", 0, 'q', 0, "quiet mode, default diagnostics suppressed" },
     { "", 0, 'v', 0, "verbose mode, expression values printed" },
     { "verbose", 0, 'V', 0, "verbose mode, annotated expression values printed" },
@@ -120,7 +125,7 @@ static pmLongOptions longopts[] = {
 
 static pmOptions opts = {
     .flags = PM_OPTFLAG_STDOUT_TZ,
-    .short_options = "a:A:bc:CdD:efFHh:j:l:m:n:O:PqS:t:T:U:vVWXxzZ:?",
+    .short_options = "a:A:bc:CdD:efFHh:j:l:m:n:o:O:PqS:t:T:U:vVWXxzZ:?",
     .long_options = longopts,
     .short_usage = "[options] [filename ...]",
     .override = override,
@@ -334,15 +339,24 @@ startmonitor(void)
     char		zero = '\0';
     char		pmie_dir[MAXPATHLEN];
 
-    /* try to create the port file directory. OK if it already exists */
+    /* try to create the port file directory. OK if it already exists
+     * - mode is 775 to match GNUmakefile
+     */
     pmsprintf(pmie_dir, sizeof(pmie_dir), "%s%c%s",
 	     pmGetConfig("PCP_TMP_DIR"), pmPathSeparator(), PMIE_SUBDIR);
-    if (mkdir2(pmie_dir, S_IRWXU | S_IRWXG | S_IRWXO) < 0) {
+    if (mkdir2(pmie_dir, S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH) < 0) {
 	if (oserror() != EEXIST) {
 	    fprintf(stderr, "%s: warning cannot create stats file dir %s: %s\n",
 		    pmGetProgname(), pmie_dir, osstrerror());
 	}
     }
+
+    /*
+     * clean the port file directory ... removes entries for pmie
+     * processes that have died and entries that cannot be valid PIDs
+     */
+    __pmCleanMapDir(pmie_dir, NULL);
+
     atexit(stopmonitor);
 
     /* create and initialize memory mapped performance data file */
@@ -485,7 +499,6 @@ sigbadproc(int sig)
 static void
 getargs(int argc, char *argv[])
 {
-    char		*configfile = NULL;
     char		*commandlog = NULL;
     char		*subopts;
     char		*subopt;
@@ -611,6 +624,10 @@ getargs(int argc, char *argv[])
 	    runfromcontrol = (strcmp(opts.optarg, "pmie_check") == 0);
 	    break;
 
+        case 'o':			/* output format for actions in archive mode */
+	    format = opts.optarg;
+	    break;
+
 	case 'U': 			/* run as named user */
 	    username = opts.optarg;
 	    isdaemon = 1;
@@ -811,7 +828,8 @@ getargs(int argc, char *argv[])
     }
     else {					/* list of 1/more filenames */
 	while (opts.optind < argc) {
-	    load(argv[opts.optind]);
+	    configfile = argv[opts.optind];
+	    load(configfile);
 	    opts.optind++;
 	}
     }
