@@ -204,7 +204,8 @@ rule_match(char *fullname, char *rulename)
     if (strcmp(fullname, rulename) == 0)
 	return 1;
     /* fullname may be a group, so match against rulename's groups */
-    s = strcpy(token, rulename);	/* reuse the token buffer */
+    pmstrncpy(token, sizeof(token), rulename);	/* reuse the token buffer */
+    s = token;
     while ((s = strrchr(s, '.')) != NULL) {
 	s[0] = '\0';
 	if (strcmp(token, fullname) == 0)
@@ -362,23 +363,23 @@ map_identifier(symbol_t *table, int tsize, int symbol_id)
 
 /* parse yes/no attribute value; returns 0 no, 1 yes, -1 error */
 int
-map_boolean(char *token)
+map_boolean(char *word)
 {
-    if (token[0] == 'y')
+    if (word[0] == 'y')
 	return 1;
-    if (token[0] == 'n')
+    if (word[0] == 'n')
 	return 0;
-    parse_error("yes or no", token);
+    parse_error("yes or no", word);
     return -1;
 }
 
 
 /* scan token from string, return 1 ok, 0 no more, -1 error */
 int
-string_token(char **scan, char *token)
+string_token(char **scan, char *word)
 {
     char	*s = *scan;
-    char	*t = token;
+    char	*t = word;
 
     while (! isgraph((int)*s) || *s == ',') {
 	if (*s == '\0')
@@ -475,6 +476,7 @@ validate(int type, char *name, char *value)
 	    ;
 	if (x < 0) {
 	    type_error(name, "include a closing single quote");
+	    free(s);
 	    return errmsg;
 	}
 	free(s);
@@ -810,23 +812,7 @@ append_string(char *s, char *append, int len)
 #endif
     if ((s = (char *)realloc(s, size)) == NULL)
 	return NULL;
-#ifdef __GNUC__
-#if __GNUC__ >= 10
-    /*
-     * gcc 10 on Fedora 32 and Debian unstable falsely report a problem
-     * with this strncat() ... it is safe
-     */
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wstringop-truncation"
-#endif
-#endif
-    strncat(s, append, len);
-#ifdef __GNUC__
-#if __GNUC__ >= 10
-#pragma GCC diagnostic pop
-#endif
-#endif
-    s[size-1] = '\0';
+    pmstrncat(s, size, append);
     return s;
 }
 
@@ -961,7 +947,7 @@ prime_next_pread(FILE *f, int end)
  * nb: `end' can be either EOL or EOF, depending on use of this routine
  */
 int
-read_token(FILE *f, char *token, int token_length, int end)
+read_token(FILE *f, char *buffer, int buffer_length, int end)
 {
     int	c;
     int	n = 0;
@@ -975,20 +961,20 @@ read_token(FILE *f, char *token, int token_length, int end)
 	while (c != '"') {
 	    if (c == '\\')
 		c = mygetc(f);
-	    if (c == end || c == EOF || n == token_length-1) {
-		token[n] = '\0';
-		parse_error("end-of-string", token);
+	    if (c == end || c == EOF || n == buffer_length-1) {
+		buffer[n] = '\0';
+		parse_error("end-of-string", buffer);
 		return -1;
 	    }
 	    if (c == '\n' && end != '\n')
 		linenum++;
-	    token[n++] = c;
+	    buffer[n++] = c;
 	    c = mygetc(f);
 	}
 	break;
     case ';':
     case '=':
-	token[n++] = c;			/* single char token */
+	buffer[n++] = c;		/* single char token */
 	break;
     default:				/* some other token */
 	while (isgraph(c)) {
@@ -996,12 +982,12 @@ read_token(FILE *f, char *token, int token_length, int end)
 		ungetc(c, f);
 		break;
 	    }
-	    if (n == token_length) {
-		token[n] = '\0';
-		parse_error("end-of-token", token);
+	    if (n == buffer_length) {
+		buffer[n] = '\0';
+		parse_error("end-of-token", buffer);
 		return -1;
 	    }
-	    token[n++] = c;
+	    buffer[n++] = c;
 	    c = mygetc(f);
 	    if (c == end || c == EOF)
 		ungetc(c, f);
@@ -1011,7 +997,7 @@ read_token(FILE *f, char *token, int token_length, int end)
 	break;
     }
 
-    token[n] = '\0';
+    buffer[n] = '\0';
     return 1;
 }
 
@@ -1848,7 +1834,7 @@ write_pmiefile(char *program, int autocreate)
 	return errmsg;
     }
     else if (!gotpath) {
-	strcpy(token, fname);
+	pmstrncpy(token, sizeof(token), fname);
 	if (realpath(token, pmiefile) == NULL) {
 	    fclose(fp);
 	    pmsprintf(errmsg, sizeof(errmsg), "failed to resolve %s realpath: %s", token, osstrerror());
@@ -1979,9 +1965,9 @@ save_area_append(char *str)
 	    return NULL;
     }
     if (sa_mark == 1)
-	save_area = strcpy(save_area, str);
+	pmstrncpy(save_area, sa_size, str);
     else
-	save_area = strcat(save_area, str);
+	pmstrncat(save_area, sa_size, str);
     sa_mark += size;
     return save_area;
 }
@@ -2176,6 +2162,7 @@ read_pmiefile(char *warning, size_t warnlen)
     /* check that we have access to all components of the path */
     if ((home = strdup(tmp)) == NULL) {
 	pmsprintf(errmsg, sizeof(errmsg), "insufficient memory for pmie file parsing");
+	fclose(f);
 	return errmsg;
     }
 #ifdef IS_MINGW
@@ -2188,6 +2175,7 @@ read_pmiefile(char *warning, size_t warnlen)
 	if (access(p, F_OK) < 0) {
 	    pmsprintf(errmsg, sizeof(errmsg), "cannot access rules path component: \"%s\"", p);
 	    free(home);
+	    fclose(f);
 	    return errmsg;
 	}
 	p = strtok(NULL, rule_path_sep);
@@ -2225,7 +2213,7 @@ initialise(char *in_rules, char *in_pmie, int autocreate, char *warning, size_t
     if (in_pmie == NULL)
 	pmsprintf(pmiefile, sizeof(pmiefile), "%s\\%s", home, DEFAULT_USER_PMIE);
     else
-	strcpy(pmiefile, in_pmie);
+	pmstrncpy(pmiefile, sizeof(pmiefile), in_pmie);
     rule_path_sep = ";";
 #else
     if (getuid() == 0) {
@@ -2255,7 +2243,7 @@ initialise(char *in_rules, char *in_pmie, int autocreate, char *warning, size_t
 	if (in_pmie == NULL)
 	    pmsprintf(pmiefile, sizeof(pmiefile), "%s%c%s", home, SEP, DEFAULT_USER_PMIE);
 	else
-	    strcpy(pmiefile, in_pmie);
+	    pmstrncpy(pmiefile, sizeof(pmiefile), in_pmie);
     }
     rule_path_sep = ":";
 #endif
@@ -2264,7 +2252,7 @@ initialise(char *in_rules, char *in_pmie, int autocreate, char *warning, size_t
 	if ((p = getenv("PMIECONF_PATH")) == NULL)
 	    pmsprintf(rulepath, sizeof(rulepath), "%s%c%s", pmGetConfig("PCP_VAR_DIR"), SEP, DEFAULT_RULES);
 	else
-	    strcpy(rulepath, p);
+	    pmstrncpy(rulepath, sizeof(rulepath), p);
     }
     else
 	pmsprintf(rulepath, sizeof(rulepath), "%s", in_rules);
@@ -2344,8 +2332,10 @@ lookup_processes(int *count, char ***processes)
 	if (ptr == NULL)
 	    continue;
 	stats = (pmiestats_t *)ptr;
-	if (strcmp(stats->config, get_pmiefile()) != 0)
+	if (strcmp(stats->config, get_pmiefile()) != 0) {
+	    __pmMemoryUnmap(ptr, statbuf.st_size);
 	    continue;
+	}
 
 	size = (1 + running) * sizeof(char *);
 	if ((proc_list = (char **)realloc(proc_list, size)) == NULL
@@ -2353,10 +2343,11 @@ lookup_processes(int *count, char ***processes)
 	    pmsprintf(errmsg, sizeof(errmsg), "insufficient memory for process search");
 	    if (proc_list) free(proc_list);
 	    closedir(dirp);
-	    close(fd);
+	    __pmMemoryUnmap(ptr, statbuf.st_size);
 	    return errmsg;
 	}
 	running++;
+	__pmMemoryUnmap(ptr, statbuf.st_size);
     }
     closedir(dirp);
     *count = running;
