@@ -141,7 +141,7 @@ _lock()
     else
 	if $VERY_VERBOSE
 	then
-	    echo "Lock acquired `cat $__dir/lock` `ls -l $__dir/lock`"
+	    echo >&2 "Lock acquired `cat $__dir/lock` `ls -l $__dir/lock`"
 	fi
     fi
 
@@ -155,7 +155,7 @@ _unlock()
     if [ -f "$__dir/lock" ]
     then
 	rm -f "$__dir/lock"
-	$VERY_VERBOSE && echo "Lock released"
+	$VERY_VERBOSE && echo >&2 "Lock released"
     fi
 }
 
@@ -253,7 +253,7 @@ _get_matching_hosts()
     fi
     for _host
     do
-	$VERY_VERBOSE && echo "Looking for host $_host in class $CLASS ..."
+	$VERY_VERBOSE && echo >&2 "Looking for host $_host in class $CLASS ..."
 	rm -f $tmp/primary_seen
 	if [ "$_host" = "$LOCALHOST" ]
 	then
@@ -373,10 +373,10 @@ _get_matching_hosts()
 	done >$tmp/tmp
 	if $VERY_VERBOSE
 	then
-	    echo "Candidate control files:"
+	    echo >&2 "Candidate control files:"
 	    sed -e 's/ .*//' <$tmp/tmp \
 	    | LC_COLLATE=POSIX sort \
-	    | uniq
+	    | uniq >&2
 	fi
 	if $EXPLICIT_CLASS
 	then
@@ -396,7 +396,7 @@ _get_matching_hosts()
 		then
 		    echo "$control" default "$host" "$primary" "$socks" "$dir" "$args" >>$tmp/tmp2
 		else
-		    $VERY_VERBOSE && echo "No match for control $control host $host directory $dir class $class"
+		    $VERY_VERBOSE && echo >&2 "No match for control $control host $host directory $dir class $class"
 		fi
 	    done
 	    if [ -s $tmp/tmp2 ]
@@ -408,8 +408,8 @@ _get_matching_hosts()
 	    fi
 	    if $VERY_VERBOSE
 	    then
-		echo "Matching control files:"
-		sed -e 's/ .*//' $tmp/tmp
+		echo >&2 "Matching control files:"
+		sed >&2 -e 's/ .*//' $tmp/tmp
 	    fi
 	else
 	    # add "class" of "-" to make $tmp/tmp format the same in
@@ -477,14 +477,14 @@ _get_matching_hosts()
     then
 	if $VERY_VERBOSE
 	then
-	    echo "_get_matching_hosts results:"
-	    echo "# control class host dir"
+	    echo >&2 "_get_matching_hosts results:"
+	    echo >&2 "# control class host dir"
 	    cat $tmp/args \
 	    | while read control class host primary socks dir other
 	    do
-		echo "$control $class $host $dir"
+		echo >&2 "$control $class $host $dir"
 	    done
-	    echo "# end"
+	    echo >&2 "# end"
 	fi
     fi
 }
@@ -627,10 +627,10 @@ _check_started()
     dir="$1"
     max=600		# 1/10 of a second, so 1 minute max
     i=0
-    $VERY_VERBOSE && $PCP_ECHO_PROG $PCP_ECHO_N "Started? ""$PCP_ECHO_C"
+    $VERY_VERBOSE && $PCP_ECHO_PROG >&2 $PCP_ECHO_N "Started? ""$PCP_ECHO_C"
     while [ $i -lt $max ]
     do
-	$VERY_VERBOSE && $PCP_ECHO_PROG $PCP_ECHO_N ".""$PCP_ECHO_C"
+	$VERY_VERBOSE && $PCP_ECHO_PROG >&2 $PCP_ECHO_N ".""$PCP_ECHO_C"
 	# rebuild active pids list, then check for our $dir
 	_get_pids_by_name ${IAM} | sed -e 's/.*/^&$/' >$tmp/pids
 	pid=`_get_pid "$dir"`
@@ -640,11 +640,11 @@ _check_started()
     done
     if [ -z "$pid" ]
     then
-	$VERY_VERBOSE && $PCP_ECHO_PROG " no"
+	$VERY_VERBOSE && $PCP_ECHO_PROG >&2 " no"
 	_warning "${IAM} failed to start for host $host and directory $dir"
 	sts=1
     else
-	$VERY_VERBOSE && $PCP_ECHO_PROG " yes"
+	$VERY_VERBOSE && $PCP_ECHO_PROG >&2 " yes"
 	if $MIGRATE
 	then
 	    # Add new process to the farm service (pmlogger_farm or pmie_farm).
@@ -668,10 +668,10 @@ _check_stopped()
     dir="$1"
     max=50		# 1/10 of a second, so 5 secs max
     i=0
-    $VERY_VERBOSE && $PCP_ECHO_PROG $PCP_ECHO_N "Stopped? ""$PCP_ECHO_C"
+    $VERY_VERBOSE && $PCP_ECHO_PROG >&2 $PCP_ECHO_N "Stopped? ""$PCP_ECHO_C"
     while [ $i -lt $max ]
     do
-	$VERY_VERBOSE && $PCP_ECHO_PROG $PCP_ECHO_N ".""$PCP_ECHO_C"
+	$VERY_VERBOSE && $PCP_ECHO_PROG >&2 $PCP_ECHO_N ".""$PCP_ECHO_C"
 	# rebuild active pids list, then check for our $dir
 	_get_pids_by_name ${IAM} | sed -e 's/.*/^&$/' >$tmp/pids
 	pid=`_get_pid "$dir"`
@@ -681,11 +681,11 @@ _check_stopped()
     done
     if [ -n "$pid" ]
     then
-	$VERY_VERBOSE && $PCP_ECHO_PROG " no"
+	$VERY_VERBOSE && $PCP_ECHO_PROG >&2 " no"
 	_warning "${IAM} failed to stop for host $host and directory $dir (PID=$pid)"
 	sts=1
     else
-	$VERY_VERBOSE && $PCP_ECHO_PROG " yes"
+	$VERY_VERBOSE && $PCP_ECHO_PROG >&2 " yes"
 	sts=0
     fi
     return $sts
@@ -925,14 +925,14 @@ found == 0 && $3 == "'"$host"'" && $6 == "'"$dir"'"	{ print NR >>"'$tmp/match'";
 	then
 	    if $VERBOSE
 	    then
-		printf "$fmt" "pmcd Host" Archive Class PID State "Instance Id"
+		printf >&2 "$fmt" "pmcd Host" Archive Class PID State "Instance Id"
 	    else
 		printf "$fmt" "pmcd Host" Archive Class PID State
 	    fi
 	else
 	    if $VERBOSE
 	    then
-		printf "$fmt" "pmcd Host" Rules Evaluations Class PID State "Instance Id"
+		printf >&2 "$fmt" "pmcd Host" Rules Evaluations Class PID State "Instance Id"
 	    else
 		printf "$fmt" "pmcd Host" Rules Evaluations Class PID State
 	    fi
@@ -952,10 +952,10 @@ found == 0 && $3 == "'"$host"'" && $6 == "'"$dir"'"	{ print NR >>"'$tmp/match'";
 		    else
 			ident=`echo "$dir" | sed -e 's;/pmie.log;;' -e 's;.*/;;'`
 		    fi
-		    printf "$fmt" "$host" "$archive" "$class" "$pid" "$state" "$ident"
+		    printf >&2 "$fmt" "$host" "$archive" "$class" "$pid" "$state" "$ident"
 		    if [ "$state" = dead ]
 		    then
-			_diagnose "$host" "$dir" 
+			_diagnose >&2 "$host" "$dir" 
 		    fi
 		done
 	    else
@@ -1075,13 +1075,13 @@ state == 3 && $1 !~ /^#/		{ state = 4; part = 2 }
 					{ part = 2 }'
 	    if $VERY_VERY_VERBOSE
 	    then
-		echo "$config split ->"
+		echo >&2 "$config split ->"
 		for p in 0 1 2 3
 		do
-		    echo "--- part $p ---"
-		    [ -f $tmp/$p ] && cat $tmp/$p
+		    echo >&2 "--- part $p ---"
+		    [ -f $tmp/$p ] && cat >&2 $tmp/$p
 		done
-		echo "--- end parts ---"
+		echo >&2 "--- end parts ---"
 	    fi
 	    if [ -f $tmp/0 ]
 	    then
@@ -1163,7 +1163,7 @@ _do_cond_create()
 	    then
 		if [ "`cat $tmp/condition-true`" -gt 0 ]
 		then
-		    $VERY_VERBOSE && echo "host: $host condition true for some class, skip pmfind class"
+		    $VERY_VERBOSE && echo >&2 "host: $host condition true for some class, skip pmfind class"
 		    break
 		fi
 		continue
@@ -1191,10 +1191,10 @@ _do_cond_create()
 			    if pminfo -h "$host" "$args" >/dev/null 2>&1
 			    then
 				touch $tmp/match
-				$VERBOSE && echo "$policy: host $host exists($args) true"
+				$VERBOSE && echo >&2 "$policy: host $host exists($args) true"
 				break
 			    else
-				$VERY_VERBOSE && echo "$policy: host $host exists($args) false"
+				$VERY_VERBOSE && echo >&2 "$policy: host $host exists($args) false"
 			    fi
 			    ;;
 
@@ -1206,10 +1206,10 @@ $2 > 0	{ sts=0; exit }
 END	{ exit(sts) }'
 			    then
 				touch $tmp/match
-				$VERBOSE && echo "$policy: host $host values($args) true"
+				$VERBOSE && echo >&2 "$policy: host $host values($args) true"
 				break
 			    else
-				$VERY_VERBOSE && echo "$policy: host $host values($args) false"
+				$VERY_VERBOSE && echo >&2 "$policy: host $host values($args) false"
 			    fi
 			    ;;
 
@@ -1227,13 +1227,13 @@ END	{ exit(sts) }'
 				if [ "$val" -gt 0 ]
 				then
 				    touch $tmp/match
-				    $VERBOSE && echo "$policy: host $host condition($args) true, value $val"
+				    $VERBOSE && echo >&2 "$policy: host $host condition($args) true, value $val"
 				    break
 				else
-				    $VERY_VERBOSE && echo "$policy: host $host condition($args) false, value $val"
+				    $VERY_VERBOSE && echo >&2 "$policy: host $host condition($args) false, value $val"
 				fi
 			    else
-				$VERY_VERBOSE && echo "$policy: host $host condition($args) false, numval $numval"
+				$VERY_VERBOSE && echo >&2 "$policy: host $host condition($args) false, numval $numval"
 
 			    fi
 			    ;;
@@ -1242,9 +1242,9 @@ END	{ exit(sts) }'
 			    if echo "$host" | grep -E "$args" >/dev/null
 			    then
 				touch $tmp/match
-				$VERBOSE && echo "$policy: host $host hostname($args) true"
+				$VERBOSE && echo >&2 "$policy: host $host hostname($args) true"
 			    else
-				$VERY_VERBOSE && echo "$policy: host $host hostname($args) false"
+				$VERY_VERBOSE && echo >&2 "$policy: host $host hostname($args) false"
 				break
 			    fi
 			    ;;
@@ -1268,13 +1268,13 @@ END	{ exit(sts) }'
 		    fi
 		fi
 	    else
-		$VERY_VERBOSE && echo "$policy: no [create] section, skip class"
+		$VERY_VERBOSE && echo >&2 "$policy: no [create] section, skip class"
 	    fi
 	done
 	n=`cat $tmp/condition-true`
 	if [ "$n" -eq 0 ]
 	then
-	    $VERBOSE && _warning "no instance created for host $host"
+	    $VERBOSE && _warning >&2 "no instance created for host $host"
 	    continue
 	elif [ "$n" -eq 1 ]
 	then
@@ -1288,7 +1288,7 @@ END	{ exit(sts) }'
 	    _resolve_configs "$host" $tmp/control.*
 	    if $VERBOSE
 	    then
-		echo "--- start combined config file ---"
+		echo >&2 "--- start combined config file ---"
 		cat $tmp/config
 		echo "--- end combined config file ---"
 	    fi
@@ -1348,11 +1348,11 @@ End-of-File
 $1 == "'"$host"'"	{ print $4 }'`
 	if $VERBOSE
 	then
-	    echo "--- start control file ---"
-	    cat $tmp/control
-	    echo "--- end control file ---"
+	    echo >&2 "--- start control file ---"
+	    cat >&2 $tmp/control
+	    echo >&2 "--- end control file ---"
 	fi
-	$VERBOSE && echo "Installing control file: $CONTROLDIR/$ident"
+	$VERBOSE && echo >&2 "Installing control file: $CONTROLDIR/$ident"
 
 	$CP $tmp/control "$CONTROLDIR/$ident"
 	$RUNASPCP "$CHECKCMD $CHECKARGS -c \"$CONTROLDIR/$ident\""
@@ -1421,7 +1421,7 @@ End-of-File
 	then
 	    :
 	else
-	    $VERBOSE && echo "Adding \$version=1.1 to control file"
+	    $VERBOSE && echo >&2 "Adding \$version=1.1 to control file"
 	    echo '#DO NOT REMOVE OR EDIT THE FOLLOWING LINE' >>$tmp/control
 	    echo '$version=1.1' >>$tmp/control
 	fi
@@ -1466,11 +1466,11 @@ $1 == "'"$host"'"	{ print $4 }'`
 	else
 	    if $VERBOSE
 	    then
-		echo "--- start control file ---"
-		cat $tmp/control
-		echo "--- end control file ---"
+		echo >&2 "--- start control file ---"
+		cat >&2 $tmp/control
+		echo >&2 "--- end control file ---"
 	    fi
-	    $VERBOSE && echo "Installing control file: $CONTROLDIR/$ident"
+	    $VERBOSE && echo >&2 "Installing control file: $CONTROLDIR/$ident"
 	    $CP $tmp/control "$CONTROLDIR/$ident"
 	    $RUNASPCP "$CHECKCMD $CHECKARGS -c \"$CONTROLDIR/$ident\""
 	    dir_args="`echo "$dir" | _expand_control`"
@@ -1510,15 +1510,15 @@ $1 == "'"#!#$host"'" && ($4 == "'"$dir"'" || $4 == "'"$alt_dir"'")	{ next }
 									{ print }'
 	if cmp -s "$control" $tmp/control
 	then
-	    $VERBOSE && echo "${IAM} for host $host and directory $dir already removed from control file $control"
+	    $VERBOSE && echo >&2 "${IAM} for host $host and directory $dir already removed from control file $control"
 	else
 	    if $VERY_VERBOSE
 	    then
-		echo "Diffs for control file $control after removing host $host and directory $dir ..."
-		diff "$control" $tmp/control
+		echo >&2 "Diffs for control file $control after removing host $host and directory $dir ..."
+		diff >&2 "$control" $tmp/control
 	    elif $VERBOSE
 	    then
-		echo "Remove ${IAM} for host $host and directory $dir in control file $control"
+		echo >&2 "Remove ${IAM} for host $host and directory $dir in control file $control"
 	    fi
 	fi
 	sed -n <$tmp/control >$tmp/tmp -e '/^[^$# 	]/p'
@@ -1529,7 +1529,7 @@ $1 == "'"#!#$host"'" && ($4 == "'"$dir"'" || $4 == "'"$alt_dir"'")	{ next }
 	    #
 	    $CP $tmp/control "$control"
 	else
-	    $VERBOSE && echo "Remove control file $control"
+	    $VERBOSE && echo >&2 "Remove control file $control"
 	    $RM "$control"
 	fi
     done
@@ -1547,12 +1547,12 @@ _do_start()
     cat $tmp/args \
     | while read control class args_host primary socks args_dir args
     do
-	$VERBOSE && echo "Looking for ${IAM} using directory $args_dir ..."
+	$VERBOSE && echo >&2 "Looking for ${IAM} using directory $args_dir ..."
 	pid=`_get_pid "$args_dir"`
 	if [ -n "$pid" ]
 	then
-	    $VERBOSE && echo "${IAM} PID $pid already running for host $args_host, nothing to do"
-	    $VERBOSE && $restart && echo "Not expected for restart!"
+	    $VERBOSE && echo >&2 "${IAM} PID $pid already running for host $args_host, nothing to do"
+	    $VERBOSE && $restart && echo >&2 "Not expected for restart!"
 	    if $MIGRATE
 	    then
 		$VERBOSE && vflag="-v"
@@ -1564,9 +1564,9 @@ _do_start()
 	then
 	    if $restart
 	    then
-		echo "Not found as expected, launching new ${IAM}"
+		echo >&2 "Not found as expected, launching new ${IAM}"
 	    else
-		echo "Not found, launching new ${IAM}"
+		echo >&2 "Not found, launching new ${IAM}"
 	    fi
 	fi
 	if [ ! -f "$control" ]
@@ -1587,16 +1587,16 @@ $1 == "'"#!#$host"'" && ($4 == "'"$dir"'" || $4 == "'"$alt_dir"'")	{ sub(/^#!#/,
 	    then
 		:
 	    else
-		$VERBOSE && echo "${IAM} for host $host and directory $dir already enabled in control file $control"
+		$VERBOSE && echo >&2 "${IAM} for host $host and directory $dir already enabled in control file $control"
 	    fi
 	else
 	    if $VERY_VERBOSE
 	    then
-		echo "Diffs for control file $control after enabling host $host and directory $dir ..."
-		diff "$control" $tmp/control
+		echo >&2 "Diffs for control file $control after enabling host $host and directory $dir ..."
+		diff >&2 "$control" $tmp/control
 	    elif $VERBOSE
 	    then
-		echo "Enable ${IAM} for host $host and directory $dir in control file $control"
+		echo >&2 "Enable ${IAM} for host $host and directory $dir in control file $control"
 	    fi
 	    $CP $tmp/control "$control"
 	fi
@@ -1632,7 +1632,7 @@ _do_stop()
 	    _warning "${IAM} for host $host already stopped, nothing to do"
 	    continue
 	fi
-	$VERBOSE && echo "Looking for ${IAM} using directory $args_dir ..."
+	$VERBOSE && echo >&2 "Looking for ${IAM} using directory $args_dir ..."
 	pid=`_get_pid "$args_dir"`
 	if [ -z "$pid" ]
 	then
@@ -1640,11 +1640,24 @@ _do_stop()
 	else
 	    # $PCPQA_KILL_SIGNAL is only intended for QA tests
 	    #
-	    $VERBOSE && echo "Found PID $pid to stop using signal ${PCPQA_KILL_SIGNAL-TERM}"
+	    $VERBOSE && echo >&2 "Found PID $pid to stop using signal ${PCPQA_KILL_SIGNAL-TERM}"
 	    $KILL ${PCPQA_KILL_SIGNAL-TERM} $pid
 	    if _check_stopped "$args_dir"
 	    then
 		:
+	    elif [ -z "$PCPQA_KILL_SIGNAL" ]
+	    then
+		# Not QA and SIGTERM did not work, try a bigger hammer ...
+		#
+		$VERBOSE && echo >&2 "That didn't work, try using signal KILL"
+		$KILL KILL $pid
+		if _check_stopped "$args_dir"
+		then
+		    :
+		else
+		    echo 1 >$tmp/sts
+		    continue
+		fi
 	    else
 		echo 1 >$tmp/sts
 		continue
@@ -1664,15 +1677,15 @@ $1 == "'"$host"'" && ($4 == "'"$dir"'" || $4 == "'"$alt_dir"'")	{ $1 = "#!#" $1
 								{ print }'
 	if cmp -s "$control" $tmp/control
 	then
-	    $VERBOSE && echo "${IAM} for host $host and directory $dir already disabled in control file $control"
+	    $VERBOSE && echo >&2 "${IAM} for host $host and directory $dir already disabled in control file $control"
 	else
 	    if $VERY_VERBOSE
 	    then
-		echo "Diffs for control file $control after disabling host $host and directory $dir ..."
-		diff "$control" $tmp/control
+		echo >&2 "Diffs for control file $control after disabling host $host and directory $dir ..."
+		diff >&2 "$control" $tmp/control
 	    elif $VERBOSE
 	    then
-		echo "Disable ${IAM} for host $host and directory $dir in control file $control"
+		echo >&2 "Disable ${IAM} for host $host and directory $dir in control file $control"
 	    fi
 	    $CP $tmp/control "$control"
 	fi
@@ -1815,9 +1828,9 @@ if $VERY_VERBOSE
 then
     if $EXPLICIT_CLASS
     then
-	echo "Using class: $CLASS"
+	echo >&2 "Using class: $CLASS"
     else
-	echo "Using default class"
+	echo >&2 "Using default class"
     fi
 fi
 [ -z "$POLICY" ] && POLICY="$PCP_ETC_DIR/pcp/${IAM}/class.d/$CLASS"
@@ -1852,7 +1865,7 @@ End-of-File
 	    echo '%h n n PCP_LOG_DIR/pmie/%i/pmie.log -c ./%i.config' >>$tmp/policy
 	fi
 	POLICY=$tmp/policy
-	$VERY_VERBOSE && echo "Using default policy"
+	$VERY_VERBOSE && echo >&2 "Using default policy"
     fi
 else
     if [ ! -f "$POLICY" ]
@@ -1866,7 +1879,7 @@ else
 	    _error "policy file $POLICY not found, class $CLASS is not defined so cannot destroy"
 	fi
     fi
-    $VERY_VERBOSE && echo "Using policy: $POLICY"
+    $VERY_VERBOSE && echo >&2 "Using policy: $POLICY"
 fi
 
 FIND_ALL_HOSTS=false
