@@ -1,12 +1,12 @@
 /*
- * Copyright (c) 2014-2017 Red Hat.
+ * Copyright (c) 2014-2017,2021 Red Hat.
  * Copyright (c) 2008 Aconex.  All Rights Reserved.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation; either version 2 of the License, or (at your
  * option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
@@ -24,83 +24,23 @@ static __pmArchCtl archctl;
 static int status;
 
 /*
- * Basic log control label sanity testing, with prefix/suffix len
+ * Basic log control label sanity testing, with header/trailer len
  * checks too (these are stored as int's around the actual label).
  */
 
 int
-verify_label(__pmFILE *f, const char *file)
+verify_label(__pmFILE *f, const char *file, __pmLogLabel *lp)
 {
-    int version, magic;
-    int n, len, xpectlen = sizeof(__pmLogLabel) + 2 * sizeof(len);
+    int		sts;
 
-    /* check the prefix integer */
-    __pmFseek(f, (long)0, SEEK_SET);
-    n = (int)__pmFread(&len, 1, sizeof(len), f);
-    len = ntohl(len);
-    if (n != sizeof(len)) {
-	if (__pmFeof(f)) {
-	    fprintf(stderr, "Bad prefix sentinel read for %s: file too short\n",
-			file);
-	    status = 2;
-	}
-	else if (__pmFerror(f)) {
-	    fprintf(stderr, "Prefix sentinel read error for %s: %s\n",
-			file, osstrerror());
-	    status = 2;
-	}
-	else {
-	    fprintf(stderr, "Prefix sentinel read error for %s: read only %d\n",
-			file, n);
-	    status = 2;
-	}
-    }
-    if (len != xpectlen) {
-	fprintf(stderr, "Bad prefix sentinel value for %s: %d (%d expected)\n",
-			file, len, xpectlen);
-	status = 2;
+    if (gold)
+	__pmLogFreeLabel(lp);
+    if ((sts = __pmLogLoadLabel(f, lp)) < 0) {
+	fprintf(stderr, "%s: cannot load label record: %s\n", file, pmErrStr(sts));
+	return -1;
     }
 
-    /* check the suffix integer */
-    __pmFseek(f, (long)(xpectlen - sizeof(len)), SEEK_SET);
-    n = (int)__pmFread(&len, 1, sizeof(len), f);
-    len = ntohl(len);
-    if (n != sizeof(len)) {
-	if (__pmFeof(f)) {
-	    fprintf(stderr, "Bad suffix sentinel read for %s: file too short\n",
-			file);
-	    status = 2;
-	}
-	else if (__pmFerror(f)) {
-	    fprintf(stderr, "Suffix sentinel read error for %s: %s\n",
-			file, osstrerror());
-	    status = 2;
-	}
-	else {
-	    fprintf(stderr, "Suffix sentinel read error for %s: read only %d\n",
-			file, n);
-	    status = 2;
-	}
-    }
-    if (len != xpectlen) {
-	fprintf(stderr, "Bad suffix sentinel value for %s: %d (%d expected)\n",
-			file, len, xpectlen);
-	status = 2;
-    }
-
-    /* check the label itself */
-    magic = logctl.l_label.ill_magic & 0xffffff00;
-    version = logctl.l_label.ill_magic & 0xff;
-    if (magic != PM_LOG_MAGIC) {
-	fprintf(stderr, "Bad magic (%x) in %s\n", magic, file);
-	status = 2;
-    }
-    if (version != PM_LOG_VERS02) {
-	fprintf(stderr, "Bad version (%x) in %s\n", version, file);
-	status = 2;
-    }
-
-    return version;
+    return __pmLogVersion(&logctl);
 }
 
 /*
@@ -111,35 +51,49 @@ verify_label(__pmFILE *f, const char *file)
 void
 compare_golden(__pmFILE *f, const char *file, int sts, int warnings)
 {
-    __pmLogLabel *label = &logctl.l_label;
+    __pmLogLabel *label = &logctl.label;
 
     if (!gold) {
 	memcpy(&golden, label, sizeof(golden));
 	if ((gold = (sts >= 0)) != 0)
 	    goldfile = strdup(file);
+	memset(label, 0, sizeof(*label));	/* avoid double freeing */
     }
     else if (warnings) {
-	int version = verify_label(f, file);
+	int version = verify_label(f, file, label);
 
-	if (version != (golden.ill_magic & 0xff)) {
+	if (version < 0)
+	    return;	/* no label content was successfully read */
+
+	if (version != (golden.magic & 0xff)) {
 	    fprintf(stderr, "Mismatched version (%x/%x) between %s and %s\n",
-			    version, golden.ill_magic & 0xff, file, goldfile);
+			    version, golden.magic & 0xff, file, goldfile);
 	    status = 2;
 	}
-	if (label->ill_pid != golden.ill_pid) {
+	if (label->pid != golden.pid) {
 	    fprintf(stderr, "Mismatched PID (%d/%d) between %s and %s\n",
-			    label->ill_pid, golden.ill_pid, file, goldfile);
+			    label->pid, golden.pid, file, goldfile);
 	    status = 2;
 	}
-	if (strncmp(label->ill_hostname, golden.ill_hostname,
-			PM_LOG_MAXHOSTLEN) != 0) {
+	if (!label->hostname || !golden.hostname ||
+	    strcmp(label->hostname, golden.hostname) != 0) {
 	    fprintf(stderr, "Mismatched hostname (%s/%s) between %s and %s\n",
-		    label->ill_hostname, golden.ill_hostname, file, goldfile);
+		    label->hostname ? label->hostname : "\"\"",
+		    golden.hostname ? golden.hostname : "\"\"", file, goldfile);
 	    status = 2;
 	}
-	if (strncmp(label->ill_tz, golden.ill_tz, PM_TZ_MAXLEN) != 0) {
+	if (!label->timezone || !golden.timezone ||
+	    strcmp(label->timezone, golden.timezone) != 0) {
 	    fprintf(stderr, "Mismatched timezone (%s/%s) between %s and %s\n",
-		    label->ill_tz, golden.ill_tz, file, goldfile);
+		    label->timezone ? label->timezone : "\"\"",
+		    golden.timezone ? golden.timezone : "\"\"", file, goldfile);
+	    status = 2;
+	}
+	if (label->zoneinfo && golden.zoneinfo &&	/* optional */
+	    strcmp(label->zoneinfo, golden.zoneinfo) != 0) {
+	    fprintf(stderr, "Mismatched zoneinfo (%s/%s) between %s and %s\n",
+		    label->zoneinfo ? label->zoneinfo : "\"\"",
+		    golden.zoneinfo ? golden.zoneinfo : "\"\"", file, goldfile);
 	    status = 2;
 	}
     }
@@ -221,7 +175,7 @@ main(int argc, char *argv[])
 
 	case 'V':	/* reset magic and version numbers */
 	    version = atoi(opts.optarg);
-	    if (version != PM_LOG_VERS02) {
+	    if (version != PM_LOG_VERS02 && version != PM_LOG_VERS03) {
 		fprintf(stderr, "%s: unknown version number (%s)\n",
 			pmGetProgname(), opts.optarg);
 		opts.errors++;
@@ -258,19 +212,19 @@ main(int argc, char *argv[])
 
     if (verbose)
 	printf("Scanning for components of archive \"%s\"\n", archive);
-    if ((sts = __pmLogLoadLabel(&archctl, archive)) < 0) {
+    if ((sts = __pmLogFindOpen(&archctl, archive)) < 0) {
 	fprintf(stderr, "%s: Cannot open archive \"%s\": %s\n",
 		pmGetProgname(), archive, pmErrStr(sts));
 	exit(1);
     }
 
     archctl.ac_curvol = -1;
-    logctl.l_physend = -1;
+    logctl.physend = -1;
 
     /*
      * Read the label from each data volume, check, and report status
      */
-    for (c = logctl.l_minvol; c <= logctl.l_maxvol; c++) {
+    for (c = logctl.minvol; c <= logctl.maxvol; c++) {
 	if (verbose)
 	    printf("Checking label on data volume %d\n", c);
 	if ((sts = __pmLogChangeVol(&archctl, c)) < 0 && warnings) {
@@ -279,17 +233,19 @@ main(int argc, char *argv[])
 	}
 	pmsprintf(buffer, sizeof(buffer), "data volume %d", c);
 	compare_golden(archctl.ac_mfp, buffer, sts, warnings);
+	__pmLogFreeLabel(&logctl.label);
     }
 
-    if (logctl.l_tifp) {
+    if (logctl.tifp) {
 	if (verbose)
 	    printf("Checking label on temporal index\n");
-	if ((sts = __pmLogChkLabel(&archctl, logctl.l_tifp, &logctl.l_label,
+	if ((sts = __pmLogChkLabel(&archctl, logctl.tifp, &logctl.label,
 			           PM_LOG_VOL_TI)) < 0 && warnings) {
 	    fprintf(stderr, "Bad temporal index label: %s\n", pmErrStr(sts));
 	    status = 2;
 	}
-	compare_golden(logctl.l_tifp, "temporal index", sts, warnings);
+	compare_golden(logctl.tifp, "temporal index", sts, warnings);
+	__pmLogFreeLabel(&logctl.label);
     }
     else if (verbose) {
 	printf("No temporal index found\n");
@@ -297,39 +253,40 @@ main(int argc, char *argv[])
 
     if (verbose)
 	printf("Checking label on metadata volume\n");
-    if ((sts = __pmLogChkLabel(&archctl, logctl.l_mdfp, &logctl.l_label,
+    if ((sts = __pmLogChkLabel(&archctl, logctl.mdfp, &logctl.label,
 			       PM_LOG_VOL_META)) < 0 && warnings) {
 	fprintf(stderr, "Bad metadata volume label: %s\n", pmErrStr(sts));
 	status = 2;
     }
-    compare_golden(logctl.l_mdfp, "metadata volume", sts, warnings);
+    compare_golden(logctl.mdfp, "metadata volume", sts, warnings);
+    __pmLogFreeLabel(&logctl.label);
 
     /*
      * Now, make any modifications requested
      */
     if (!readonly) {
 	if (version)
-	    golden.ill_magic = PM_LOG_MAGIC | version;
+	    golden.magic = PM_LOG_MAGIC | version;
 	if (pid)
-	    golden.ill_pid = pid;
+	    golden.pid = pid;
 	if (host) {
-	    memset(golden.ill_hostname, 0, sizeof(golden.ill_hostname));
-	    strncpy(golden.ill_hostname, host, PM_LOG_MAXHOSTLEN-1);
-	    golden.ill_hostname[PM_LOG_MAXHOSTLEN-1] = '\0';
+	    free(golden.hostname);
+	    golden.hostname = strdup(host);
 	}
 	if (tz) {
-	    memset(golden.ill_tz, 0, sizeof(golden.ill_tz));
-	    strncpy(golden.ill_tz, tz, PM_TZ_MAXLEN-1);
-	    golden.ill_tz[PM_TZ_MAXLEN-1] = '\0';
+	    free(golden.timezone);
+	    golden.timezone = strdup(tz);
 	}
+	/* TODO: v3 archive zoneinfo */
+	golden.zoneinfo = NULL;
 
 	if (archctl.ac_mfp)
 	    __pmFclose(archctl.ac_mfp);
-	for (c = logctl.l_minvol; c <= logctl.l_maxvol; c++) {
+	for (c = logctl.minvol; c <= logctl.maxvol; c++) {
 	    if (verbose)
 		printf("Writing label on data volume %d\n", c);
-	    golden.ill_vol = c;
-	    pmsprintf(buffer, sizeof(buffer), "%s.%d", logctl.l_name, c);
+	    golden.vol = c;
+	    pmsprintf(buffer, sizeof(buffer), "%s.%d", logctl.name, c);
 	    if ((archctl.ac_mfp = __pmFopen(buffer, "r+")) == NULL) {
 		fprintf(stderr, "Failed data volume %d open: %s\n",
 				c, osstrerror());
@@ -346,41 +303,45 @@ main(int argc, char *argv[])
 	/* Need to reset the data volume, for subsequent label read */
 	archctl.ac_mfp = NULL;
 	archctl.ac_curvol = -1;
-	__pmLogChangeVol(&archctl, logctl.l_minvol);
+	__pmLogChangeVol(&archctl, logctl.minvol);
 
-	if (logctl.l_tifp) {
-	    __pmFclose(logctl.l_tifp);
+	if (logctl.tifp) {
+	    __pmFclose(logctl.tifp);
 	    if (verbose)
 		printf("Writing label on temporal index\n");
-	    golden.ill_vol = PM_LOG_VOL_TI;
-	    pmsprintf(buffer, sizeof(buffer), "%s.index", logctl.l_name);
-	    if ((logctl.l_tifp = __pmFopen(buffer, "r+")) == NULL) {
+	    golden.vol = PM_LOG_VOL_TI;
+	    pmsprintf(buffer, sizeof(buffer), "%s.index", logctl.name);
+	    if ((logctl.tifp = __pmFopen(buffer, "r+")) == NULL) {
 		fprintf(stderr, "Failed temporal index open: %s\n",
 				osstrerror());
 		status = 3;
 	    }
-	    else if ((sts = __pmLogWriteLabel(logctl.l_tifp, &golden)) < 0) {
+	    else if ((sts = __pmLogWriteLabel(logctl.tifp, &golden)) < 0) {
 		fprintf(stderr, "Failed temporal index label write: %s\n",
 				pmErrStr(sts));
 		status = 3;
 	    }
+	    if (logctl.tifp)
+		__pmFclose(logctl.tifp);
 	}
 
-	__pmFclose(logctl.l_mdfp);
+	__pmFclose(logctl.mdfp);
 	if (verbose)
 	    printf("Writing label on metadata volume\n");
-	golden.ill_vol = PM_LOG_VOL_META;
-	pmsprintf(buffer, sizeof(buffer), "%s.meta", logctl.l_name);
-	if ((logctl.l_mdfp = __pmFopen(buffer, "r+")) == NULL) {
+	golden.vol = PM_LOG_VOL_META;
+	pmsprintf(buffer, sizeof(buffer), "%s.meta", logctl.name);
+	if ((logctl.mdfp = __pmFopen(buffer, "r+")) == NULL) {
 	    fprintf(stderr, "Failed metadata volume open: %s\n",
 			    osstrerror());
 	    status = 3;
 	}
-	else if ((sts = __pmLogWriteLabel(logctl.l_mdfp, &golden)) < 0) {
+	else if ((sts = __pmLogWriteLabel(logctl.mdfp, &golden)) < 0) {
 	    fprintf(stderr, "Failed metadata volume label write: %s\n",
 			    pmErrStr(sts));
 	    status = 3;
 	}
+	if (logctl.mdfp)
+	    __pmFclose(logctl.mdfp);
     }
 
     /*
@@ -390,22 +351,22 @@ main(int argc, char *argv[])
 	char	       *ddmm;
 	char	       *yr;
 	struct timeval	tv;
-	time_t t = golden.ill_start.tv_sec;
+	time_t t = golden.start.sec;
 
-	printf("Log Label (Log Format Version %d)\n", golden.ill_magic & 0xff);
-	printf("Performance metrics from host %s\n", golden.ill_hostname);
+	printf("Log Label (Log Format Version %d)\n", golden.magic & 0xff);
+	printf("Performance metrics from host %s\n", golden.hostname);
 
 	ddmm = pmCtime(&t, buffer);
 	ddmm[10] = '\0';
 	yr = &ddmm[20];
 	printf("  commencing %s ", ddmm);
-	tv.tv_sec = golden.ill_start.tv_sec;
-	tv.tv_usec = golden.ill_start.tv_usec;
-	pmPrintStamp(stdout, &tv);
+	tv.tv_sec = golden.start.sec;
+	tv.tv_usec = golden.start.nsec / 1000;
+	pmPrintStamp(stdout, &tv);	/* TODO: v3 archives */
 	printf(" %4.4s\n", yr);
 	if ((sts = pmNewContext(PM_CONTEXT_ARCHIVE, archive)) < 0)
 	    printf("  ending     UNKNOWN (pmNewContext: %s)\n", pmErrStr(sts));
-	else if ((sts = pmGetArchiveEnd(&tv)) < 0)
+	else if ((sts = pmGetArchiveEnd(&tv)) < 0)	/* TODO: v3 archives */
 	    printf("  ending     UNKNOWN (pmGetArchiveEnd: %s)\n", pmErrStr(sts));
 	else {
 	    time_t	time;
@@ -414,12 +375,12 @@ main(int argc, char *argv[])
 	    ddmm[10] = '\0';
 	    yr = &ddmm[20];
 	    printf("  ending     %s ", ddmm);
-	    pmPrintStamp(stdout, &tv);
+	    pmPrintStamp(stdout, &tv);	/* TODO: v3 archives */
 	    printf(" %4.4s\n", yr);
 	}
-	if (Lflag) {
-	    printf("Archive timezone: %s\n", golden.ill_tz);
-	    printf("PID for pmlogger: %d\n", golden.ill_pid);
+	if (Lflag) {	/* TODO: v3 archives - zoneinfo */
+	    printf("Archive timezone: %s\n", golden.timezone);
+	    printf("PID for pmlogger: %d\n", golden.pid);
 	}
     }
 
