@@ -1,7 +1,8 @@
 #include "pmlogreduce.h"
+#include "pcp/archive.h"
 
-void
-doindom(pmResult *rp)
+int
+doindom(__pmResult *rp)
 {
     pmValueSet		*vsp;
     int			i;
@@ -12,7 +13,6 @@ doindom(pmResult *rp)
     int			*instlist;
     char		**names;
     int			sts;
-    __pmTimestamp	stamp;
 
     for (i = 0; i < rp->numpmid; i++) {
 	vsp = rp->vset[i];
@@ -34,8 +34,8 @@ doindom(pmResult *rp)
 	    fprintf(stderr,
 		"%s: doindom: Arrgh, unexpected PMID %s @ vset[%d]\n",
 		    pmGetProgname(), pmIDStr(vsp->pmid), i);
-	    __pmDumpResult(stderr, rp);
-	    exit(1);
+	    __pmPrintResult(stderr, rp);
+	    return PM_ERR_GENERIC;
 	}
 	if (mp->idp == NULL)
 	    continue;
@@ -44,7 +44,7 @@ doindom(pmResult *rp)
 	    fprintf(stderr,
 		"%s: doindom: pmGetInDom (%s) failed: %s\n",
 		    pmGetProgname(), pmInDomStr(mp->idp->indom), pmErrStr(sts));
-	    exit(1);
+	    return sts;
 	}
 
 	need = 1;
@@ -71,25 +71,41 @@ doindom(pmResult *rp)
 	}
 
 	if (need) {
+	    __pmLogInDom	lid;
+	    int			pdu_type;
 	    if (pmDebugOptions.appl0) {
 		fprintf(stderr, "Add metadata: indom %s for metric %s\n", pmInDomStr(mp->idp->indom), pmIDStr(vsp->pmid));
 	    }
 	    if (mp->idp->name != NULL) free(mp->idp->name);
 	    if (mp->idp->inst != NULL) free(mp->idp->inst);
-	    mp->idp->name = names;
-	    mp->idp->inst = instlist;
-	    mp->idp->numinst = sts;
-#if 0	    // TODO use current when it => __pmTimestamp
-	    if ((sts = __pmLogPutInDom(&archctl, mp->idp->indom, &current, mp->idp->numinst, mp->idp->inst, mp->idp->name)) < 0) {
-#else
-	    stamp.sec = current.tv_sec;
-	    stamp.nsec = current.tv_usec * 1000;
-	    if ((sts = __pmLogPutInDom(&archctl, mp->idp->indom, &stamp, mp->idp->numinst, mp->idp->inst, mp->idp->name)) < 0) {
-#endif
+	    lid.indom = mp->idp->indom;
+	    lid.stamp = current;	/* struct assignment */
+	    lid.numinst = mp->idp->numinst = sts;
+	    lid.instlist = mp->idp->inst = instlist;
+	    lid.namelist = mp->idp->name = names;
+	    lid.alloc = 0;
+	    if (__pmLogVersion(archctl.ac_log) >= PM_LOG_VERS03) {
+		/* try delta indom */
+		pdu_type = TYPE_INDOM;
+		sts = pmaTryDeltaInDom(archctl.ac_log, NULL, &lid);
+		if (sts < 0) {
+		    fprintf(stderr, "Botch: pmaTryDeltaInDom failed: %d\n", sts);
+		    return PM_ERR_GENERIC;
+
+		}
+		if (sts == 1)
+		    pdu_type = TYPE_INDOM_DELTA;
+	    }
+	    else
+		pdu_type = TYPE_INDOM_V2;
+	    sts = __pmLogPutInDom(&archctl, pdu_type, &lid);
+	    if (pdu_type == TYPE_INDOM_DELTA)
+		__pmFreeLogInDom(&lid);
+	    if (sts < 0) {
 		fprintf(stderr,
 		    "%s: Error: failed to add pmInDom: indom %s (for pmid %s): %s\n",
 			pmGetProgname(), pmInDomStr(mp->idp->indom), pmIDStr(vsp->pmid), pmErrStr(sts));
-		exit(1);
+		return sts;
 	    }
 	    needti = 1;		/* requires a temporal index update */
 	}
@@ -100,10 +116,5 @@ doindom(pmResult *rp)
 
     }
 
-    if (needti) {
-	__pmFflush(logctl.mdfp);
-	stamp.sec = current.tv_sec;
-	stamp.nsec = current.tv_usec * 1000;
-	__pmLogPutIndex(&archctl, &stamp);
-    }
+    return needti;
 }
