@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018-2019,2021-2023 Red Hat.
+ * Copyright (c) 2018-2019,2021-2024 Red Hat.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -37,7 +37,7 @@
 typedef enum proxy_registry {
     METRICS_NOTUSED	= 0,	/* special "next available" MMV cluster */
     METRICS_SERVER,
-    METRICS_REDIS,
+    METRICS_KEYS,
     METRICS_HTTP,
     METRICS_PCP,
     METRICS_DISCOVER,
@@ -84,15 +84,11 @@ typedef struct stream {
 typedef enum stream_protocol {
     STREAM_UNKNOWN	= 0,
     STREAM_SECURE	= 0x1,
-    STREAM_REDIS	= 0x2,
+    STREAM_KEYS		= 0x2,
     STREAM_HTTP		= 0x4,
     STREAM_PCP		= 0x8,
 } stream_protocol_t;
 
-typedef struct redis_client {
-    redisReader		*reader;	/* RESP request handling state */
-} redis_client_t;
-
 typedef struct http_client {
     http_parser		parser;		/* HTTP request parsing state */
     struct servlet	*servlet;	/* servicing current request */
@@ -110,6 +106,10 @@ typedef struct http_client {
 #endif
 } http_client_t;
 
+typedef struct key_client {
+    respReader		*reader;	/* RESP request handling state */
+} key_client_t;
+
 typedef struct pcp_client {
     pcp_proxy_state_t	state;
     unsigned int	port : 16;
@@ -146,8 +146,8 @@ typedef struct client {
     secure_client	secure;
 #endif
     union {
-	redis_client_t	redis;
 	http_client_t	http;
+	key_client_t	keys;
 	pcp_client_t	pcp;
     } u;
     struct proxy	*proxy;
@@ -163,13 +163,13 @@ typedef struct proxy {
     struct client	*first;		/* doubly linked list of clients */
     struct server	*servers;	/* array of tcp/pipe socket servers */
     unsigned int	nservers;	/* count of entries in server array */
-    unsigned int	redisetup;	/* is Redis slots information setup */
+    unsigned int	keys_setup;	/* key server slot information setup */
     struct client	*pending_writes;
 #ifdef HAVE_OPENSSL
     SSL_CTX		*ssl;
     __pmSecureConfig	tls;
 #endif
-    redisSlots		*slots;		/* mapping of Redis keys to servers */
+    keySlots		*slots;		/* mapping of key names to servers */
     struct servlet	*servlets;	/* linked list of http URL handlers */
     mmv_registry_t	*metrics[NUM_REGISTRY];	/* performance metrics */
     pmAtomValue		*values[NUM_VALUES]; /* local metric values */
@@ -209,16 +209,16 @@ extern void on_secure_client_close(struct client *);
 #define on_secure_client_close(c)	do { (void)(c); } while (0)
 #endif
 
-extern void on_redis_client_read(struct proxy *, struct client *,
-				ssize_t, const uv_buf_t *);
-extern void on_redis_client_write(struct client *);
-extern void on_redis_client_close(struct client *);
-
 extern void on_http_client_read(struct proxy *, struct client *,
 				ssize_t, const uv_buf_t *);
 extern void on_http_client_write(struct client *);
 extern void on_http_client_close(struct client *);
 
+extern void on_key_client_read(struct proxy *, struct client *,
+				ssize_t, const uv_buf_t *);
+extern void on_key_client_write(struct client *);
+extern void on_key_client_close(struct client *);
+
 extern void on_pcp_client_read(struct proxy *, struct client *,
 				ssize_t, const uv_buf_t *);
 extern void on_pcp_client_write(struct client *);
@@ -234,12 +234,12 @@ extern void close_secure_module(struct proxy *);
 #define close_secure_module(p)	do { (void)(p); } while (0)
 #endif
 
-extern void setup_redis_module(struct proxy *);
-extern void close_redis_module(struct proxy *);
-
 extern void setup_http_module(struct proxy *);
 extern void close_http_module(struct proxy *);
 
+extern void setup_keys_module(struct proxy *);
+extern void close_keys_module(struct proxy *);
+
 extern void setup_pcp_module(struct proxy *);
 extern void close_pcp_module(struct proxy *);
 
