@@ -1,6 +1,6 @@
 """Wrapper module for libpcp_import - Performace Co-Pilot Log Import API
 #
-# Copyright (C) 2012-2021 Red Hat.
+# Copyright (C) 2012-2022 Red Hat.
 #
 # This file is part of the "pcp" module, the python interfaces for the
 # Performance Co-Pilot toolkit.
@@ -18,7 +18,6 @@
 
 # Example use of this module for creating a PCP archive:
 
-        import math
         import time
         import pmapi
         from pcp import pmi
@@ -43,21 +42,23 @@
         log.pmiPutValue("kernel.all.load", "1 minute", "%f" % 0.01)
         log.pmiPutValue("kernel.all.load", "5 minute", "%f" % 0.05)
         log.pmiPutValue("kernel.all.load", "15 minute", "%f" % 0.15)
-        timetuple = math.modf(time.time())
-        useconds = int(timetuple[0] * 1000000)
-        seconds = int(timetuple[1])
-        log.pmiWrite(seconds, useconds)
+        log.pmiWrite(time.time())  # sec since epoch, or datetime, or
+        #log.pmiWrite(seconds, useconds)
+
         del log
 """
+# pylint: disable=too-many-arguments,too-many-positional-arguments
 
-from pcp.pmapi import pmID, pmInDom, pmUnits, pmResult
+from pcp.pmapi import pmID, pmInDom, pmUnits, pmHighResResult, pmResult
 from cpmi import pmiErrSymDict, PMI_MAXERRMSGLEN
-
-import ctypes
-from ctypes import cast, c_int, c_uint, c_char_p, POINTER
+from ctypes import c_int, c_uint, c_longlong, c_char_p
+from ctypes import cast, create_string_buffer, POINTER, CDLL
+from ctypes.util import find_library
+from datetime import datetime
+from math import modf
 
 # Performance Co-Pilot PMI library (C)
-LIBPCP_IMPORT = ctypes.CDLL(ctypes.util.find_library("pcp_import"))
+LIBPCP_IMPORT = CDLL(find_library("pcp_import"))
 
 ##
 # PMI Log Import Services
@@ -97,6 +98,9 @@ LIBPCP_IMPORT.pmiSetHostname.argtypes = [c_char_p]
 LIBPCP_IMPORT.pmiSetTimezone.restype = c_int
 LIBPCP_IMPORT.pmiSetTimezone.argtypes = [c_char_p]
 
+LIBPCP_IMPORT.pmiSetVersion.restype = c_int
+LIBPCP_IMPORT.pmiSetVersion.argtypes = [c_int]
+
 LIBPCP_IMPORT.pmiAddMetric.restype = c_int
 LIBPCP_IMPORT.pmiAddMetric.argtypes = [
         c_char_p, pmID, c_int, pmInDom, c_int, pmUnits]
@@ -113,8 +117,14 @@ LIBPCP_IMPORT.pmiGetHandle.argtypes = [c_char_p, c_char_p]
 LIBPCP_IMPORT.pmiPutValueHandle.restype = c_int
 LIBPCP_IMPORT.pmiPutValueHandle.argtypes = [c_int, c_char_p]
 
-LIBPCP_IMPORT.pmiWrite.restype = c_int
-LIBPCP_IMPORT.pmiWrite.argtypes = [c_int, c_int]
+LIBPCP_IMPORT.pmiWrite2.restype = c_int
+LIBPCP_IMPORT.pmiWrite2.argtypes = [c_longlong, c_int]
+
+LIBPCP_IMPORT.pmiHighResWrite.restype = c_int
+LIBPCP_IMPORT.pmiHighResWrite.argtypes = [c_longlong, c_int]
+
+LIBPCP_IMPORT.pmiPutHighResResult.restype = c_int
+LIBPCP_IMPORT.pmiPutHighResResult.argtypes = [POINTER(pmHighResResult)]
 
 LIBPCP_IMPORT.pmiPutResult.restype = c_int
 LIBPCP_IMPORT.pmiPutResult.argtypes = [POINTER(pmResult)]
@@ -147,7 +157,7 @@ class pmiErr(Exception):
     def __str__(self):
         try:
             error_symbol = pmiErrSymDict[self.code]
-            error_string = ctypes.create_string_buffer(PMI_MAXERRMSGLEN)
+            error_string = create_string_buffer(PMI_MAXERRMSGLEN)
             error_string = LIBPCP_IMPORT.pmiErrStr_r(self.code, error_string,
                                                      PMI_MAXERRMSGLEN)
         except KeyError:
@@ -192,6 +202,7 @@ class pmiLogImport(object):
         if not isinstance(path, bytes):
             path = path.encode('utf-8')
         self._path = path        # the archive path (file name)
+        self._epoch = datetime.utcfromtimestamp(0)    # epoch sec
         self._ctx = LIBPCP_IMPORT.pmiStart(c_char_p(path), inherit)
         if self._ctx < 0:
             raise pmiErr(self._ctx)
@@ -230,6 +241,19 @@ class pmiLogImport(object):
             raise pmiErr(status)
         return status
 
+    def pmiSetVersion(self, version):
+        """PMI - set the output archive version (2 or 3)
+        """
+        status = LIBPCP_IMPORT.pmiUseContext(self._ctx)
+        if status < 0:
+            raise pmiErr(status)
+        if not isinstance(version, int):
+            version = 2
+        status = LIBPCP_IMPORT.pmiSetVersion(version)
+        if status < 0:
+            raise pmiErr(status)
+        return status
+
     @staticmethod
     def pmiID(domain, cluster, item):
         """PMI - construct a pmID data structure (helper routine) """
@@ -248,13 +272,11 @@ class pmiLogImport(object):
     @staticmethod
     def pmiUnits(dim_space, dim_time, dim_count,
                  scale_space, scale_time, scale_count):
-        # pylint: disable=R0913
         """PMI - construct a pmiUnits data structure (helper routine) """
         return LIBPCP_IMPORT.pmiUnits(dim_space, dim_time, dim_count,
                                       scale_space, scale_time, scale_count)
 
     def pmiAddMetric(self, name, pmid, typed, indom, sem, units):
-        # pylint: disable=R0913
         """PMI - add a new metric definition to a Log Import context """
         status = LIBPCP_IMPORT.pmiUseContext(self._ctx)
         if status < 0:
@@ -328,12 +350,31 @@ class pmiLogImport(object):
             raise pmiErr(status)
         return status
 
-    def pmiWrite(self, sec, usec):
+    def pmiHighResWrite(self, sec, nsec):
         """PMI - flush data to a Log Import archive """
         status = LIBPCP_IMPORT.pmiUseContext(self._ctx)
         if status < 0:
             raise pmiErr(status)
-        status = LIBPCP_IMPORT.pmiWrite(sec, usec)
+        status = LIBPCP_IMPORT.pmiHighResWrite(sec, nsec)
+        if status < 0:
+            raise pmiErr(status)
+        return status
+
+    def pmiWrite(self, sec, usec=None):
+        """PMI - flush data to a Log Import archive """
+        status = LIBPCP_IMPORT.pmiUseContext(self._ctx)
+        if status < 0:
+            raise pmiErr(status)
+        if sec and not usec:
+            if isinstance(sec, datetime):
+                sec = float((sec - self._epoch).total_seconds())
+            if isinstance(sec, float):
+                ts = modf(sec)
+                sec = int(ts[1])
+                usec = int(ts[0] * 1000000)
+            else:
+                usec = 0
+        status = LIBPCP_IMPORT.pmiWrite2(sec, usec)
         if status < 0:
             raise pmiErr(status)
         return status
@@ -358,6 +399,16 @@ class pmiLogImport(object):
             raise pmiErr(status)
         return status
 
+    def put_highres_result(self, result):
+        """PMI - add a data record to a Log Import archive """
+        status = LIBPCP_IMPORT.pmiUseContext(self._ctx)
+        if status < 0:
+            raise pmiErr(status)
+        status = LIBPCP_IMPORT.pmiPutHighResResult(cast(result, POINTER(pmHighResResult)))
+        if status < 0:
+            raise pmiErr(status)
+        return status
+
     def pmiPutText(self, typ, cls, ident, content):
         """PMI - add a text record to a Log Import archive """
         status = LIBPCP_IMPORT.pmiUseContext(self._ctx)
@@ -371,7 +422,6 @@ class pmiLogImport(object):
         return status
 
     def pmiPutLabel(self, typ, ident, inst, name, content):
-        # pylint: disable=R0913
         """PMI - add a label record to a Log Import archive """
         status = LIBPCP_IMPORT.pmiUseContext(self._ctx)
         if status < 0:
